@@ -1,12 +1,9 @@
-#[macro_use]
-extern crate criterion;
-use criterion::Criterion;
+use criterion::{criterion_group, criterion_main, Criterion};
 
 use arrow2::array::*;
 use arrow2::util::bench_util::*;
 use arrow2::{
-    compute::arithmetics::basic::add, compute::arithmetics::basic::div_scalar, datatypes::DataType,
-    types::NativeType,
+    compute::arithmetics::basic::add, compute::arithmetics::basic::div_scalar, types::NativeType,
 };
 use num_traits::NumCast;
 use std::ops::{Add, Div};
@@ -22,14 +19,14 @@ fn bench_add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>)
 where
     T: NativeType + Add<Output = T> + NumCast,
 {
-    criterion::black_box(add(lhs, rhs)).unwrap();
+    criterion::black_box(add(lhs, rhs));
 }
 
 fn add_benchmark(c: &mut Criterion) {
     (10..=20).step_by(2).for_each(|log2_size| {
         let size = 2usize.pow(log2_size);
-        let arr_a = create_primitive_array_with_seed::<u64>(size, DataType::UInt64, 0.0, 43);
-        let arr_b = create_primitive_array_with_seed::<u64>(size, DataType::UInt64, 0.0, 42);
+        let arr_a = create_primitive_array_with_seed::<u64>(size, 0.0, 43);
+        let arr_b = create_primitive_array_with_seed::<u64>(size, 0.0, 42);
 
         c.bench_function(&format!("divide_scalar 2^{}", log2_size), |b| {
             // 4 is a very fast optimizable divisor
