@@ -92,6 +92,19 @@ impl<O: Offset> MutableBinaryArray<O> {
         }
     }
 
+    /// Initializes a new [`MutableBinaryArray`] with a pre-allocated capacity of slots and values.
+    pub fn with_capacities(capacity: usize, values: usize) -> Self {
+        let mut offsets = MutableBuffer::<O>::with_capacity(capacity + 1);
+        offsets.push(O::default());
+
+        Self {
+            data_type: Self::default_data_type(),
+            offsets,
+            values: MutableBuffer::<u8>::with_capacity(values),
+            validity: None,
+        }
+    }
+
     /// Reserves `additional` slots.
     pub fn reserve(&mut self, additional: usize) {
         self.offsets.reserve(additional);
@@ -123,11 +136,10 @@ impl<O: Offset> MutableBinaryArray<O> {
     }
 
     fn init_validity(&mut self) {
-        self.validity = Some(MutableBitmap::from_trusted_len_iter(
-            std::iter::repeat(true)
-                .take(self.len() - 1)
-                .chain(std::iter::once(false)),
-        ))
+        let mut validity = MutableBitmap::with_capacity(self.offsets.capacity());
+        validity.extend_constant(self.len(), true);
+        validity.set(self.len() - 1, false);
+        self.validity = Some(validity)
     }
 
     /// Converts itself into an [`Array`].
@@ -145,6 +157,18 @@ impl<O: Offset> MutableBinaryArray<O> {
     }
 }
 
+impl<O: Offset> MutableBinaryArray<O> {
+    /// returns its values.
+    pub fn values(&self) -> &MutableBuffer<u8> {
+        &self.values
+    }
+
+    /// returns its offsets.
+    pub fn offsets(&self) -> &MutableBuffer<O> {
+        &self.offsets
+    }
+}
+
 impl<O: Offset> MutableArray for MutableBinaryArray<O> {
     fn len(&self) -> usize {
         self.offsets.len() - 1
@@ -227,14 +251,25 @@ impl<O: Offset> MutableBinaryArray<O> {
         unsafe { Self::from_trusted_len_iter_unchecked(iterator) }
     }
 
+    /// Creates a new [`BinaryArray`] from a [`TrustedLen`] of `&[u8]`.
+    /// # Safety
+    /// The iterator must be [`TrustedLen`](https://doc.rust-lang.org/std/iter/trait.TrustedLen.html).
+    /// I.e. that `size_hint().1` correctly reports its length.
+    #[inline]
+    pub unsafe fn from_trusted_len_values_iter_unchecked<T: AsRef<[u8]>, I: Iterator<Item = T>>(
+        iterator: I,
+    ) -> Self {
+        let (offsets, values) = unsafe { trusted_len_values_iter(iterator) };
+        Self::from_data(Self::default_data_type(), offsets, values, None)
+    }
+
     /// Creates a new [`BinaryArray`] from a [`TrustedLen`] of `&[u8]`.
     #[inline]
     pub fn from_trusted_len_values_iter<T: AsRef<[u8]>, I: TrustedLen<Item = T>>(
         iterator: I,
     ) -> Self {
         // soundness: I is `TrustedLen`
-        let (offsets, values) = unsafe { trusted_len_values_iter(iterator) };
-        Self::from_data(Self::default_data_type(), offsets, values, None)
+        unsafe { Self::from_trusted_len_values_iter_unchecked(iterator) }
     }
 
     /// Creates a [`MutableBinaryArray`] from an falible iterator of trusted length.
@@ -252,7 +287,11 @@ impl<O: Offset> MutableBinaryArray<O> {
         let iterator = iterator.into_iter();
 
         // soundness: assumed trusted len
-        let (validity, offsets, values) = try_trusted_len_unzip(iterator)?;
+        let (mut validity, offsets, values) = try_trusted_len_unzip(iterator)?;
+
+        if validity.as_mut().unwrap().null_count() == 0 {
+            validity = None;
+        }
 
         Ok(Self::from_data(
             Self::default_data_type(),
@@ -371,8 +410,8 @@ impl<O: Offset, T: AsRef<[u8]>> TryPush<Option<T>> for MutableBinaryArray<O> {
             Some(value) => {
                 let bytes = value.as_ref();
 
-                let size = O::from_usize(self.values.len() + bytes.len())
-                    .ok_or(ArrowError::KeyOverflowError)?;
+                let size =
+                    O::from_usize(self.values.len() + bytes.len()).ok_or(ArrowError::Overflow)?;
 
                 self.values.extend_from_slice(bytes);
 
