@@ -11,26 +11,26 @@ unsafe impl<K: DictionaryKey> ToFfi for DictionaryArray<K> {
         self.keys.buffers()
     }
 
-    #[inline]
-    fn offset(&self) -> usize {
-        self.offset
+    fn offset(&self) -> Option<usize> {
+        self.keys.offset()
+    }
+
+    fn to_ffi_aligned(&self) -> Self {
+        Self {
+            data_type: self.data_type.clone(),
+            keys: self.keys.to_ffi_aligned(),
+            values: self.values.clone(),
+        }
     }
 }
 
 impl<K: DictionaryKey, A: ffi::ArrowArrayRef> FromFfi<A> for DictionaryArray<K> {
     unsafe fn try_from_ffi(array: A) -> Result<Self> {
         // keys: similar to PrimitiveArray, but the datatype is the inner one
-        let length = array.array().len();
-        let offset = array.array().offset();
-        let mut validity = unsafe { array.validity() }?;
-        let mut values = unsafe { array.buffer::<K>(0) }?;
+        let validity = unsafe { array.validity() }?;
+        let values = unsafe { array.buffer::<K>(1) }?;
 
-        if offset > 0 {
-            values = values.slice(offset, length);
-            validity = validity.map(|x| x.slice(offset, length))
-        }
         let keys = PrimitiveArray::<K>::from_data(K::DATA_TYPE, values, validity);
-        // values
         let values = array.dictionary()?.unwrap();
         let values = ffi::try_from(values)?.into();
 
