@@ -1,7 +1,8 @@
 use crate::{bitmap::Bitmap, buffer::Buffer, datatypes::DataType, error::Result};
 
-use super::{display_fmt, display_helper, ffi::ToFfi, Array};
+use super::{display_fmt, Array};
 
+mod ffi;
 mod iterator;
 mod mutable;
 pub use mutable::*;
@@ -14,7 +15,6 @@ pub struct FixedSizeBinaryArray {
     data_type: DataType,
     values: Buffer<u8>,
     validity: Option<Bitmap>,
-    offset: usize,
 }
 
 impl FixedSizeBinaryArray {
@@ -34,7 +34,7 @@ impl FixedSizeBinaryArray {
 
     /// Returns a new [`FixedSizeBinaryArray`].
     pub fn from_data(data_type: DataType, values: Buffer<u8>, validity: Option<Bitmap>) -> Self {
-        let size = *Self::get_size(&data_type) as usize;
+        let size = Self::get_size(&data_type);
 
         assert_eq!(values.len() % size, 0);
 
@@ -47,7 +47,6 @@ impl FixedSizeBinaryArray {
             data_type,
             values,
             validity,
-            offset: 0,
         }
     }
 
@@ -83,10 +82,30 @@ impl FixedSizeBinaryArray {
             size: self.size,
             values,
             validity,
-            offset: self.offset + offset,
         }
     }
 
+    /// Sets the validity bitmap on this [`FixedSizeBinaryArray`].
+    /// # Panic
+    /// This function panics iff `validity.len() != self.len()`.
+    pub fn with_validity(&self, validity: Option<Bitmap>) -> Self {
+        if matches!(&validity, Some(bitmap) if bitmap.len() != self.len()) {
+            panic!("validity should be as least as large as the array")
+        }
+        let mut arr = self.clone();
+        arr.validity = validity;
+        arr
+    }
+}
+
+// accessors
+impl FixedSizeBinaryArray {
+    /// Returns the length of this array
+    #[inline]
+    pub fn len(&self) -> usize {
+        self.values.len() / self.size as usize
+    }
+
     /// The optional validity.
     #[inline]
     pub fn validity(&self) -> Option<&Bitmap> {
@@ -103,7 +122,8 @@ impl FixedSizeBinaryArray {
     /// Panics iff `i >= self.len()`.
     #[inline]
     pub fn value(&self, i: usize) -> &[u8] {
-        &self.values()[i * self.size as usize..(i + 1) * self.size as usize]
+        assert!(i < self.len());
+        unsafe { self.value_unchecked(i) }
     }
 
     /// Returns the element at index `i` as &str
@@ -116,28 +136,39 @@ impl FixedSizeBinaryArray {
             .get_unchecked(i * self.size..(i + 1) * self.size)
     }
 
+    /// Returns a new [`FixedSizeBinary`] with a different logical type.
+    /// This is `O(1)`.
+    /// # Panics
+    /// Panics iff the data_type is not supported for the physical type.
+    #[inline]
+    pub fn to(self, data_type: DataType) -> Self {
+        match (
+            data_type.to_logical_type(),
+            self.data_type().to_logical_type(),
+        ) {
+            (DataType::FixedSizeBinary(size_a), DataType::FixedSizeBinary(size_b))
+                if size_a == size_b => {}
+            _ => panic!("Wrong DataType"),
+        }
+
+        Self {
+            size: self.size,
+            data_type,
+            values: self.values,
+            validity: self.validity,
+        }
+    }
+
     /// Returns the size
     pub fn size(&self) -> usize {
         self.size
     }
-
-    /// Sets the validity bitmap on this [`FixedSizeBinaryArray`].
-    /// # Panic
-    /// This function panics iff `validity.len() != self.len()`.
-    pub fn with_validity(&self, validity: Option<Bitmap>) -> Self {
-        if matches!(&validity, Some(bitmap) if bitmap.len() != self.len()) {
-            panic!("validity should be as least as large as the array")
-        }
-        let mut arr = self.clone();
-        arr.validity = validity;
-        arr
-    }
 }
 
 impl FixedSizeBinaryArray {
-    pub(crate) fn get_size(data_type: &DataType) -> &i32 {
+    pub(crate) fn get_size(data_type: &DataType) -> usize {
         match data_type.to_logical_type() {
-            DataType::FixedSizeBinary(size) => size,
+            DataType::FixedSizeBinary(size) => *size,
             _ => panic!("Wrong DataType"),
         }
     }
@@ -151,7 +182,7 @@ impl Array for FixedSizeBinaryArray {
 
     #[inline]
     fn len(&self) -> usize {
-        self.values.len() / self.size as usize
+        self.len()
     }
 
     #[inline]
@@ -176,25 +207,11 @@ impl Array for FixedSizeBinaryArray {
 
 impl std::fmt::Display for FixedSizeBinaryArray {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let a = |x: &[u8]| display_helper(x.iter().map(|x| Some(format!("{:b}", x)))).join(" ");
-        let iter = self.iter().map(|x| x.map(a));
+        let iter = self.iter().map(|x| x.map(|x| format!("{:?}", x)));
         display_fmt(iter, "FixedSizeBinaryArray", f, false)
     }
 }
 
-unsafe impl ToFfi for FixedSizeBinaryArray {
-    fn buffers(&self) -> Vec<Option<std::ptr::NonNull<u8>>> {
-        vec![
-            self.validity.as_ref().map(|x| x.as_ptr()),
-            std::ptr::NonNull::new(self.values.as_ptr() as *mut u8),
-        ]
-    }
-
-    fn offset(&self) -> usize {
-        self.offset
-    }
-}
-
 impl FixedSizeBinaryArray {
     /// Creates a [`FixedSizeBinaryArray`] from an fallible iterator of optional `[u8]`.
     pub fn try_from_iter<P: AsRef<[u8]>, I: IntoIterator<Item = Option<P>>>(
