@@ -5,8 +5,9 @@ use crate::{
     datatypes::{DataType, Field},
 };
 
-use super::{display_fmt, ffi::ToFfi, new_empty_array, new_null_array, Array};
+use super::{display_fmt, new_empty_array, new_null_array, Array};
 
+mod ffi;
 mod iterator;
 pub use iterator::*;
 mod mutable;
@@ -20,7 +21,6 @@ pub struct FixedSizeListArray {
     data_type: DataType,
     values: Arc<dyn Array>,
     validity: Option<Bitmap>,
-    offset: usize,
 }
 
 impl FixedSizeListArray {
@@ -60,7 +60,6 @@ impl FixedSizeListArray {
             data_type,
             values,
             validity,
-            offset: 0,
         }
     }
 
@@ -97,10 +96,30 @@ impl FixedSizeListArray {
             size: self.size,
             values,
             validity,
-            offset: self.offset + offset,
         }
     }
 
+    /// Sets the validity bitmap on this [`FixedSizeListArray`].
+    /// # Panic
+    /// This function panics iff `validity.len() != self.len()`.
+    pub fn with_validity(&self, validity: Option<Bitmap>) -> Self {
+        if matches!(&validity, Some(bitmap) if bitmap.len() != self.len()) {
+            panic!("validity should be as least as large as the array")
+        }
+        let mut arr = self.clone();
+        arr.validity = validity;
+        arr
+    }
+}
+
+// accessors
+impl FixedSizeListArray {
+    /// Returns the length of this array
+    #[inline]
+    pub fn len(&self) -> usize {
+        self.values.len() / self.size as usize
+    }
+
     /// The optional validity.
     #[inline]
     pub fn validity(&self) -> Option<&Bitmap> {
@@ -129,18 +148,6 @@ impl FixedSizeListArray {
         self.values
             .slice_unchecked(i * self.size as usize, self.size as usize)
     }
-
-    /// Sets the validity bitmap on this [`FixedSizeListArray`].
-    /// # Panic
-    /// This function panics iff `validity.len() != self.len()`.
-    pub fn with_validity(&self, validity: Option<Bitmap>) -> Self {
-        if matches!(&validity, Some(bitmap) if bitmap.len() != self.len()) {
-            panic!("validity should be as least as large as the array")
-        }
-        let mut arr = self.clone();
-        arr.validity = validity;
-        arr
-    }
 }
 
 impl FixedSizeListArray {
@@ -154,7 +161,7 @@ impl FixedSizeListArray {
     /// Returns a [`DataType`] consistent with [`FixedSizeListArray`].
     pub fn default_datatype(data_type: DataType, size: usize) -> DataType {
         let field = Box::new(Field::new("item", data_type, true));
-        DataType::FixedSizeList(field, size as i32)
+        DataType::FixedSizeList(field, size)
     }
 }
 
@@ -166,7 +173,7 @@ impl Array for FixedSizeListArray {
 
     #[inline]
     fn len(&self) -> usize {
-        self.values.len() / self.size as usize
+        self.len()
     }
 
     #[inline]
@@ -194,17 +201,3 @@ impl std::fmt::Display for FixedSizeListArray {
         display_fmt(self.iter(), "FixedSizeListArray", f, true)
     }
 }
-
-unsafe impl ToFfi for FixedSizeListArray {
-    fn buffers(&self) -> Vec<Option<std::ptr::NonNull<u8>>> {
-        vec![self.validity.as_ref().map(|x| x.as_ptr())]
-    }
-
-    fn offset(&self) -> usize {
-        self.offset
-    }
-
-    fn children(&self) -> Vec<Arc<dyn Array>> {
-        vec![self.values().clone()]
-    }
-}
