@@ -3,20 +3,18 @@ use std::ops::Add;
 
 use num_traits::{ops::overflowing::OverflowingAdd, CheckedAdd, SaturatingAdd, WrappingAdd, Zero};
 
-use crate::compute::arithmetics::basic::check_same_type;
-use crate::compute::arithmetics::ArrayWrappingAdd;
 use crate::{
     array::{Array, PrimitiveArray},
     bitmap::Bitmap,
     compute::{
         arithmetics::{
-            ArrayAdd, ArrayCheckedAdd, ArrayOverflowingAdd, ArraySaturatingAdd, NotI128,
+            ArrayAdd, ArrayCheckedAdd, ArrayOverflowingAdd, ArraySaturatingAdd, ArrayWrappingAdd,
+            NativeArithmetics,
         },
         arity::{
             binary, binary_checked, binary_with_bitmap, unary, unary_checked, unary_with_bitmap,
         },
     },
-    error::Result,
     types::NativeType,
 };
 
@@ -30,16 +28,14 @@ use crate::{
 ///
 /// let a = PrimitiveArray::from([None, Some(6), None, Some(6)]);
 /// let b = PrimitiveArray::from([Some(5), None, None, Some(6)]);
-/// let result = add(&a, &b).unwrap();
+/// let result = add(&a, &b);
 /// let expected = PrimitiveArray::from([None, None, None, Some(12)]);
 /// assert_eq!(result, expected)
 /// ```
-pub fn add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + Add<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     binary(lhs, rhs, lhs.data_type().clone(), |a, b| a + b)
 }
 
@@ -53,19 +49,14 @@ where
 ///
 /// let a = PrimitiveArray::from([Some(-100i8), Some(100i8), Some(100i8)]);
 /// let b = PrimitiveArray::from([Some(0i8), Some(100i8), Some(0i8)]);
-/// let result = wrapping_add(&a, &b).unwrap();
+/// let result = wrapping_add(&a, &b);
 /// let expected = PrimitiveArray::from([Some(-100i8), Some(-56i8), Some(100i8)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn wrapping_add<T>(
-    lhs: &PrimitiveArray<T>,
-    rhs: &PrimitiveArray<T>,
-) -> Result<PrimitiveArray<T>>
+pub fn wrapping_add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + WrappingAdd<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.wrapping_add(&b);
 
     binary(lhs, rhs, lhs.data_type().clone(), op)
@@ -81,16 +72,14 @@ where
 ///
 /// let a = PrimitiveArray::from([Some(100i8), Some(100i8), Some(100i8)]);
 /// let b = PrimitiveArray::from([Some(0i8), Some(100i8), Some(0i8)]);
-/// let result = checked_add(&a, &b).unwrap();
+/// let result = checked_add(&a, &b);
 /// let expected = PrimitiveArray::from([Some(100i8), None, Some(100i8)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn checked_add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn checked_add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
-    T: NativeType + CheckedAdd<Output = T> + Zero,
+    T: NativeType + CheckedAdd<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.checked_add(&b);
 
     binary_checked(lhs, rhs, lhs.data_type().clone(), op)
@@ -107,19 +96,14 @@ where
 ///
 /// let a = PrimitiveArray::from([Some(100i8)]);
 /// let b = PrimitiveArray::from([Some(100i8)]);
-/// let result = saturating_add(&a, &b).unwrap();
+/// let result = saturating_add(&a, &b);
 /// let expected = PrimitiveArray::from([Some(127)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn saturating_add<T>(
-    lhs: &PrimitiveArray<T>,
-    rhs: &PrimitiveArray<T>,
-) -> Result<PrimitiveArray<T>>
+pub fn saturating_add<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + SaturatingAdd<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.saturating_add(&b);
 
     binary(lhs, rhs, lhs.data_type().clone(), op)
@@ -137,19 +121,17 @@ where
 ///
 /// let a = PrimitiveArray::from([Some(1i8), Some(100i8)]);
 /// let b = PrimitiveArray::from([Some(1i8), Some(100i8)]);
-/// let (result, overflow) = overflowing_add(&a, &b).unwrap();
+/// let (result, overflow) = overflowing_add(&a, &b);
 /// let expected = PrimitiveArray::from([Some(2i8), Some(-56i8)]);
 /// assert_eq!(result, expected);
 /// ```
 pub fn overflowing_add<T>(
     lhs: &PrimitiveArray<T>,
     rhs: &PrimitiveArray<T>,
-) -> Result<(PrimitiveArray<T>, Bitmap)>
+) -> (PrimitiveArray<T>, Bitmap)
 where
     T: NativeType + OverflowingAdd<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.overflowing_add(&b);
 
     binary_with_bitmap(lhs, rhs, lhs.data_type().clone(), op)
@@ -158,22 +140,18 @@ where
 // Implementation of ArrayAdd trait for PrimitiveArrays
 impl<T> ArrayAdd<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + Add<Output = T> + NotI128,
+    T: NativeArithmetics + Add<Output = T>,
 {
-    type Output = Self;
-
-    fn add(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn add(&self, rhs: &PrimitiveArray<T>) -> Self {
         add(self, rhs)
     }
 }
 
 impl<T> ArrayWrappingAdd<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + WrappingAdd<Output = T> + NotI128,
+    T: NativeArithmetics + WrappingAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn wrapping_add(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn wrapping_add(&self, rhs: &PrimitiveArray<T>) -> Self {
         wrapping_add(self, rhs)
     }
 }
@@ -181,11 +159,9 @@ where
 // Implementation of ArrayCheckedAdd trait for PrimitiveArrays
 impl<T> ArrayCheckedAdd<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedAdd<Output = T> + Zero + NotI128,
+    T: NativeArithmetics + CheckedAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn checked_add(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn checked_add(&self, rhs: &PrimitiveArray<T>) -> Self {
         checked_add(self, rhs)
     }
 }
@@ -193,11 +169,9 @@ where
 // Implementation of ArraySaturatingAdd trait for PrimitiveArrays
 impl<T> ArraySaturatingAdd<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + SaturatingAdd<Output = T> + NotI128,
+    T: NativeArithmetics + SaturatingAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn saturating_add(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn saturating_add(&self, rhs: &PrimitiveArray<T>) -> Self {
         saturating_add(self, rhs)
     }
 }
@@ -205,11 +179,9 @@ where
 // Implementation of ArraySaturatingAdd trait for PrimitiveArrays
 impl<T> ArrayOverflowingAdd<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + OverflowingAdd<Output = T> + NotI128,
+    T: NativeArithmetics + OverflowingAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn overflowing_add(&self, rhs: &PrimitiveArray<T>) -> Result<(Self::Output, Bitmap)> {
+    fn overflowing_add(&self, rhs: &PrimitiveArray<T>) -> (Self, Bitmap) {
         overflowing_add(self, rhs)
     }
 }
@@ -271,7 +243,7 @@ where
 /// ```
 pub fn checked_add_scalar<T>(lhs: &PrimitiveArray<T>, rhs: &T) -> PrimitiveArray<T>
 where
-    T: NativeType + CheckedAdd<Output = T> + Zero,
+    T: NativeType + CheckedAdd<Output = T>,
 {
     let rhs = *rhs;
     let op = move |a: T| a.checked_add(&rhs);
@@ -331,47 +303,39 @@ where
 // Implementation of ArrayAdd trait for PrimitiveArrays with a scalar
 impl<T> ArrayAdd<T> for PrimitiveArray<T>
 where
-    T: NativeType + Add<Output = T> + NotI128,
+    T: NativeArithmetics + Add<Output = T>,
 {
-    type Output = Self;
-
-    fn add(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(add_scalar(self, rhs))
+    fn add(&self, rhs: &T) -> Self {
+        add_scalar(self, rhs)
     }
 }
 
 // Implementation of ArrayCheckedAdd trait for PrimitiveArrays with a scalar
 impl<T> ArrayCheckedAdd<T> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedAdd<Output = T> + Zero + NotI128,
+    T: NativeArithmetics + CheckedAdd<Output = T> + Zero,
 {
-    type Output = Self;
-
-    fn checked_add(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(checked_add_scalar(self, rhs))
+    fn checked_add(&self, rhs: &T) -> Self {
+        checked_add_scalar(self, rhs)
     }
 }
 
 // Implementation of ArraySaturatingAdd trait for PrimitiveArrays with a scalar
 impl<T> ArraySaturatingAdd<T> for PrimitiveArray<T>
 where
-    T: NativeType + SaturatingAdd<Output = T> + NotI128,
+    T: NativeArithmetics + SaturatingAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn saturating_add(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(saturating_add_scalar(self, rhs))
+    fn saturating_add(&self, rhs: &T) -> Self {
+        saturating_add_scalar(self, rhs)
     }
 }
 
 // Implementation of ArraySaturatingAdd trait for PrimitiveArrays with a scalar
 impl<T> ArrayOverflowingAdd<T> for PrimitiveArray<T>
 where
-    T: NativeType + OverflowingAdd<Output = T> + NotI128,
+    T: NativeArithmetics + OverflowingAdd<Output = T>,
 {
-    type Output = Self;
-
-    fn overflowing_add(&self, rhs: &T) -> Result<(Self::Output, Bitmap)> {
-        Ok(overflowing_add_scalar(self, rhs))
+    fn overflowing_add(&self, rhs: &T) -> (Self, Bitmap) {
+        overflowing_add_scalar(self, rhs)
     }
 }
