@@ -3,15 +3,14 @@ use std::ops::Div;
 
 use num_traits::{CheckedDiv, NumCast, Zero};
 
-use crate::compute::arithmetics::basic::{check_same_len, check_same_type};
 use crate::datatypes::DataType;
 use crate::{
     array::{Array, PrimitiveArray},
     compute::{
-        arithmetics::{ArrayCheckedDiv, ArrayDiv, NotI128},
+        arithmetics::{ArrayCheckedDiv, ArrayDiv, NativeArithmetics},
         arity::{binary, binary_checked, unary, unary_checked},
+        utils::check_same_len,
     },
-    error::Result,
     types::NativeType,
 };
 use strength_reduce::{
@@ -28,26 +27,24 @@ use strength_reduce::{
 ///
 /// let a = Int32Array::from(&[Some(10), Some(1), Some(6)]);
 /// let b = Int32Array::from(&[Some(5), None, Some(6)]);
-/// let result = div(&a, &b).unwrap();
+/// let result = div(&a, &b);
 /// let expected = Int32Array::from(&[Some(2), None, Some(1)]);
 /// assert_eq!(result, expected)
 /// ```
-pub fn div<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn div<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + Div<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     if rhs.null_count() == 0 {
         binary(lhs, rhs, lhs.data_type().clone(), |a, b| a / b)
     } else {
-        check_same_len(lhs, rhs)?;
+        check_same_len(lhs, rhs).unwrap();
         let values = lhs.iter().zip(rhs.iter()).map(|(l, r)| match (l, r) {
             (Some(l), Some(r)) => Some(*l / *r),
             _ => None,
         });
 
-        Ok(PrimitiveArray::from_trusted_len_iter(values).to(lhs.data_type().clone()))
+        PrimitiveArray::from_trusted_len_iter(values).to(lhs.data_type().clone())
     }
 }
 
@@ -62,16 +59,14 @@ where
 ///
 /// let a = Int8Array::from(&[Some(-100i8), Some(10i8)]);
 /// let b = Int8Array::from(&[Some(100i8), Some(0i8)]);
-/// let result = checked_div(&a, &b).unwrap();
+/// let result = checked_div(&a, &b);
 /// let expected = Int8Array::from(&[Some(-1i8), None]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn checked_div<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn checked_div<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
-    T: NativeType + CheckedDiv<Output = T> + Zero,
+    T: NativeArithmetics + CheckedDiv<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.checked_div(&b);
 
     binary_checked(lhs, rhs, lhs.data_type().clone(), op)
@@ -80,11 +75,9 @@ where
 // Implementation of ArrayDiv trait for PrimitiveArrays
 impl<T> ArrayDiv<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + Div<Output = T> + NotI128,
+    T: NativeArithmetics + Div<Output = T>,
 {
-    type Output = Self;
-
-    fn div(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn div(&self, rhs: &PrimitiveArray<T>) -> Self {
         div(self, rhs)
     }
 }
@@ -92,11 +85,9 @@ where
 // Implementation of ArrayCheckedDiv trait for PrimitiveArrays
 impl<T> ArrayCheckedDiv<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedDiv<Output = T> + Zero + NotI128,
+    T: NativeArithmetics + CheckedDiv<Output = T>,
 {
-    type Output = Self;
-
-    fn checked_div(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn checked_div(&self, rhs: &PrimitiveArray<T>) -> Self {
         checked_div(self, rhs)
     }
 }
@@ -210,23 +201,19 @@ where
 // Implementation of ArrayDiv trait for PrimitiveArrays with a scalar
 impl<T> ArrayDiv<T> for PrimitiveArray<T>
 where
-    T: NativeType + Div<Output = T> + NotI128 + NumCast,
+    T: NativeType + Div<Output = T> + NativeArithmetics + NumCast,
 {
-    type Output = Self;
-
-    fn div(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(div_scalar(self, rhs))
+    fn div(&self, rhs: &T) -> Self {
+        div_scalar(self, rhs)
     }
 }
 
 // Implementation of ArrayCheckedDiv trait for PrimitiveArrays with a scalar
 impl<T> ArrayCheckedDiv<T> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedDiv<Output = T> + Zero + NotI128,
+    T: NativeType + CheckedDiv<Output = T> + Zero + NativeArithmetics,
 {
-    type Output = Self;
-
-    fn checked_div(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(checked_div_scalar(self, rhs))
+    fn checked_div(&self, rhs: &T) -> Self {
+        checked_div_scalar(self, rhs)
     }
 }
