@@ -1,22 +1,20 @@
 //! Definition of basic mul operations with primitive arrays
 use std::ops::Mul;
 
-use num_traits::{ops::overflowing::OverflowingMul, CheckedMul, SaturatingMul, WrappingMul, Zero};
+use num_traits::{ops::overflowing::OverflowingMul, CheckedMul, SaturatingMul, WrappingMul};
 
-use crate::compute::arithmetics::basic::check_same_type;
-use crate::compute::arithmetics::ArrayWrappingMul;
 use crate::{
     array::{Array, PrimitiveArray},
     bitmap::Bitmap,
     compute::{
         arithmetics::{
-            ArrayCheckedMul, ArrayMul, ArrayOverflowingMul, ArraySaturatingMul, NotI128,
+            ArrayCheckedMul, ArrayMul, ArrayOverflowingMul, ArraySaturatingMul, ArrayWrappingMul,
+            NativeArithmetics,
         },
         arity::{
             binary, binary_checked, binary_with_bitmap, unary, unary_checked, unary_with_bitmap,
         },
     },
-    error::Result,
     types::NativeType,
 };
 
@@ -30,16 +28,14 @@ use crate::{
 ///
 /// let a = Int32Array::from(&[None, Some(6), None, Some(6)]);
 /// let b = Int32Array::from(&[Some(5), None, None, Some(6)]);
-/// let result = mul(&a, &b).unwrap();
+/// let result = mul(&a, &b);
 /// let expected = Int32Array::from(&[None, None, None, Some(36)]);
 /// assert_eq!(result, expected)
 /// ```
-pub fn mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + Mul<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     binary(lhs, rhs, lhs.data_type().clone(), |a, b| a * b)
 }
 
@@ -53,19 +49,14 @@ where
 ///
 /// let a = PrimitiveArray::from([Some(100i8), Some(0x10i8), Some(100i8)]);
 /// let b = PrimitiveArray::from([Some(0i8), Some(0x10i8), Some(0i8)]);
-/// let result = wrapping_mul(&a, &b).unwrap();
+/// let result = wrapping_mul(&a, &b);
 /// let expected = PrimitiveArray::from([Some(0), Some(0), Some(0)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn wrapping_mul<T>(
-    lhs: &PrimitiveArray<T>,
-    rhs: &PrimitiveArray<T>,
-) -> Result<PrimitiveArray<T>>
+pub fn wrapping_mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + WrappingMul<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.wrapping_mul(&b);
 
     binary(lhs, rhs, lhs.data_type().clone(), op)
@@ -82,16 +73,14 @@ where
 ///
 /// let a = Int8Array::from(&[Some(100i8), Some(100i8), Some(100i8)]);
 /// let b = Int8Array::from(&[Some(1i8), Some(100i8), Some(1i8)]);
-/// let result = checked_mul(&a, &b).unwrap();
+/// let result = checked_mul(&a, &b);
 /// let expected = Int8Array::from(&[Some(100i8), None, Some(100i8)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn checked_mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> Result<PrimitiveArray<T>>
+pub fn checked_mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
-    T: NativeType + CheckedMul<Output = T> + Zero,
+    T: NativeType + CheckedMul<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.checked_mul(&b);
 
     binary_checked(lhs, rhs, lhs.data_type().clone(), op)
@@ -108,19 +97,14 @@ where
 ///
 /// let a = Int8Array::from(&[Some(-100i8)]);
 /// let b = Int8Array::from(&[Some(100i8)]);
-/// let result = saturating_mul(&a, &b).unwrap();
+/// let result = saturating_mul(&a, &b);
 /// let expected = Int8Array::from(&[Some(-128)]);
 /// assert_eq!(result, expected);
 /// ```
-pub fn saturating_mul<T>(
-    lhs: &PrimitiveArray<T>,
-    rhs: &PrimitiveArray<T>,
-) -> Result<PrimitiveArray<T>>
+pub fn saturating_mul<T>(lhs: &PrimitiveArray<T>, rhs: &PrimitiveArray<T>) -> PrimitiveArray<T>
 where
     T: NativeType + SaturatingMul<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.saturating_mul(&b);
 
     binary(lhs, rhs, lhs.data_type().clone(), op)
@@ -138,19 +122,17 @@ where
 ///
 /// let a = Int8Array::from(&[Some(1i8), Some(-100i8)]);
 /// let b = Int8Array::from(&[Some(1i8), Some(100i8)]);
-/// let (result, overflow) = overflowing_mul(&a, &b).unwrap();
+/// let (result, overflow) = overflowing_mul(&a, &b);
 /// let expected = Int8Array::from(&[Some(1i8), Some(-16i8)]);
 /// assert_eq!(result, expected);
 /// ```
 pub fn overflowing_mul<T>(
     lhs: &PrimitiveArray<T>,
     rhs: &PrimitiveArray<T>,
-) -> Result<(PrimitiveArray<T>, Bitmap)>
+) -> (PrimitiveArray<T>, Bitmap)
 where
     T: NativeType + OverflowingMul<Output = T>,
 {
-    check_same_type(lhs, rhs)?;
-
     let op = move |a: T, b: T| a.overflowing_mul(&b);
 
     binary_with_bitmap(lhs, rhs, lhs.data_type().clone(), op)
@@ -159,22 +141,18 @@ where
 // Implementation of ArrayMul trait for PrimitiveArrays
 impl<T> ArrayMul<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + Mul<Output = T> + NotI128,
+    T: NativeArithmetics + Mul<Output = T>,
 {
-    type Output = Self;
-
-    fn mul(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn mul(&self, rhs: &PrimitiveArray<T>) -> Self {
         mul(self, rhs)
     }
 }
 
 impl<T> ArrayWrappingMul<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + WrappingMul<Output = T> + NotI128,
+    T: NativeArithmetics + WrappingMul<Output = T>,
 {
-    type Output = Self;
-
-    fn wrapping_mul(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn wrapping_mul(&self, rhs: &PrimitiveArray<T>) -> Self {
         wrapping_mul(self, rhs)
     }
 }
@@ -182,11 +160,9 @@ where
 // Implementation of ArrayCheckedMul trait for PrimitiveArrays
 impl<T> ArrayCheckedMul<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedMul<Output = T> + Zero + NotI128,
+    T: NativeArithmetics + CheckedMul<Output = T>,
 {
-    type Output = Self;
-
-    fn checked_mul(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn checked_mul(&self, rhs: &PrimitiveArray<T>) -> Self {
         checked_mul(self, rhs)
     }
 }
@@ -194,11 +170,9 @@ where
 // Implementation of ArraySaturatingMul trait for PrimitiveArrays
 impl<T> ArraySaturatingMul<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + SaturatingMul<Output = T> + NotI128,
+    T: NativeArithmetics + SaturatingMul<Output = T>,
 {
-    type Output = Self;
-
-    fn saturating_mul(&self, rhs: &PrimitiveArray<T>) -> Result<Self::Output> {
+    fn saturating_mul(&self, rhs: &PrimitiveArray<T>) -> Self {
         saturating_mul(self, rhs)
     }
 }
@@ -206,14 +180,13 @@ where
 // Implementation of ArraySaturatingMul trait for PrimitiveArrays
 impl<T> ArrayOverflowingMul<PrimitiveArray<T>> for PrimitiveArray<T>
 where
-    T: NativeType + OverflowingMul<Output = T> + NotI128,
+    T: NativeArithmetics + OverflowingMul<Output = T>,
 {
-    type Output = Self;
-
-    fn overflowing_mul(&self, rhs: &PrimitiveArray<T>) -> Result<(Self::Output, Bitmap)> {
+    fn overflowing_mul(&self, rhs: &PrimitiveArray<T>) -> (Self, Bitmap) {
         overflowing_mul(self, rhs)
     }
 }
+
 /// Multiply a scalar T to a primitive array of type T.
 /// Panics if the multiplication of the values overflows.
 ///
@@ -271,7 +244,7 @@ where
 /// ```
 pub fn checked_mul_scalar<T>(lhs: &PrimitiveArray<T>, rhs: &T) -> PrimitiveArray<T>
 where
-    T: NativeType + CheckedMul<Output = T> + Zero,
+    T: NativeType + CheckedMul<Output = T>,
 {
     let rhs = *rhs;
     let op = move |a: T| a.checked_mul(&rhs);
@@ -331,47 +304,39 @@ where
 // Implementation of ArrayMul trait for PrimitiveArrays with a scalar
 impl<T> ArrayMul<T> for PrimitiveArray<T>
 where
-    T: NativeType + Mul<Output = T> + NotI128,
+    T: NativeType + Mul<Output = T> + NativeArithmetics,
 {
-    type Output = Self;
-
-    fn mul(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(mul_scalar(self, rhs))
+    fn mul(&self, rhs: &T) -> Self {
+        mul_scalar(self, rhs)
     }
 }
 
 // Implementation of ArrayCheckedMul trait for PrimitiveArrays with a scalar
 impl<T> ArrayCheckedMul<T> for PrimitiveArray<T>
 where
-    T: NativeType + CheckedMul<Output = T> + Zero + NotI128,
+    T: NativeArithmetics + CheckedMul<Output = T>,
 {
-    type Output = Self;
-
-    fn checked_mul(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(checked_mul_scalar(self, rhs))
+    fn checked_mul(&self, rhs: &T) -> Self {
+        checked_mul_scalar(self, rhs)
     }
 }
 
 // Implementation of ArraySaturatingMul trait for PrimitiveArrays with a scalar
 impl<T> ArraySaturatingMul<T> for PrimitiveArray<T>
 where
-    T: NativeType + SaturatingMul<Output = T> + NotI128,
+    T: NativeArithmetics + SaturatingMul<Output = T>,
 {
-    type Output = Self;
-
-    fn saturating_mul(&self, rhs: &T) -> Result<Self::Output> {
-        Ok(saturating_mul_scalar(self, rhs))
+    fn saturating_mul(&self, rhs: &T) -> Self {
+        saturating_mul_scalar(self, rhs)
     }
 }
 
 // Implementation of ArraySaturatingMul trait for PrimitiveArrays with a scalar
 impl<T> ArrayOverflowingMul<T> for PrimitiveArray<T>
 where
-    T: NativeType + OverflowingMul<Output = T> + NotI128,
+    T: NativeArithmetics + OverflowingMul<Output = T>,
 {
-    type Output = Self;
-
-    fn overflowing_mul(&self, rhs: &T) -> Result<(Self::Output, Bitmap)> {
-        Ok(overflowing_mul_scalar(self, rhs))
+    fn overflowing_mul(&self, rhs: &T) -> (Self, Bitmap) {
+        overflowing_mul_scalar(self, rhs)
     }
 }
