@@ -124,7 +124,11 @@ pub fn is_null(input: &dyn Array) -> BooleanArray {
 /// ```
 pub fn is_not_null(input: &dyn Array) -> BooleanArray {
     let values = match input.validity() {
-        None => Bitmap::from_trusted_len_iter(std::iter::repeat(true).take(input.len())),
+        None => {
+            let mut mutable = MutableBitmap::new();
+            mutable.extend_constant(input.len(), true);
+            mutable.into()
+        }
         Some(buffer) => buffer.clone(),
     };
     BooleanArray::from_data(DataType::Boolean, values, None)
