@@ -1,7 +1,7 @@
 //! Contains the operator [`if_then_else`].
-use crate::array::growable;
+use crate::array::{growable, Array, BooleanArray};
+use crate::bitmap::utils::SlicesIterator;
 use crate::error::{ArrowError, Result};
-use crate::{array::*, bitmap::utils::SlicesIterator};
 
 /// Returns the values from `lhs` if the predicate is `true` or from the `lhs` if the predicate is false
 /// Returns `None` if the predicate is `None`.
