@@ -23,7 +23,7 @@ use super::DataType;
 
 /// A logical [`DataType`] and its associated metadata per
 /// [Arrow specification](https://arrow.apache.org/docs/cpp/api/datatype.html)
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, Eq)]
 pub struct Field {
     /// Its name
     pub name: String,
@@ -39,6 +39,26 @@ pub struct Field {
     pub metadata: Option<BTreeMap<String, String>>,
 }
 
+impl std::hash::Hash for Field {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.name.hash(state);
+        self.data_type.hash(state);
+        self.nullable.hash(state);
+        self.dict_is_ordered.hash(state);
+        self.metadata.hash(state);
+    }
+}
+
+impl PartialEq for Field {
+    fn eq(&self, other: &Self) -> bool {
+        self.name == other.name
+            && self.data_type == other.data_type
+            && self.nullable == other.nullable
+            && self.dict_is_ordered == other.dict_is_ordered
+            && self.metadata == other.metadata
+    }
+}
+
 impl Field {
     /// Creates a new field
     pub fn new(name: &str, data_type: DataType, nullable: bool) -> Self {
@@ -157,7 +177,7 @@ impl Field {
                 for (key, from_value) in from_metadata {
                     if let Some(self_value) = self_metadata.get(key) {
                         if self_value != from_value {
-                            return Err(ArrowError::Schema(format!(
+                            return Err(ArrowError::InvalidArgumentError(format!(
                                 "Fail to merge field due to conflicting metadata data value for key {}", key),
                             ));
                         }
@@ -173,12 +193,12 @@ impl Field {
             _ => {}
         }
         if from.dict_id != self.dict_id {
-            return Err(ArrowError::Schema(
+            return Err(ArrowError::InvalidArgumentError(
                 "Fail to merge schema Field due to conflicting dict_id".to_string(),
             ));
         }
         if from.dict_is_ordered != self.dict_is_ordered {
-            return Err(ArrowError::Schema(
+            return Err(ArrowError::InvalidArgumentError(
                 "Fail to merge schema Field due to conflicting dict_is_ordered".to_string(),
             ));
         }
@@ -200,7 +220,7 @@ impl Field {
                     }
                 }
                 _ => {
-                    return Err(ArrowError::Schema(
+                    return Err(ArrowError::InvalidArgumentError(
                         "Fail to merge schema Field due to conflicting datatype".to_string(),
                     ));
                 }
@@ -221,7 +241,7 @@ impl Field {
                     }
                 }
                 _ => {
-                    return Err(ArrowError::Schema(
+                    return Err(ArrowError::InvalidArgumentError(
                         "Fail to merge schema Field due to conflicting datatype".to_string(),
                     ));
                 }
@@ -259,7 +279,7 @@ impl Field {
             | DataType::Map(_, _)
             | DataType::Decimal(_, _) => {
                 if self.data_type != from.data_type {
-                    return Err(ArrowError::Schema(
+                    return Err(ArrowError::InvalidArgumentError(
                         "Fail to merge schema Field due to conflicting datatype".to_string(),
                     ));
                 }
