@@ -74,7 +74,7 @@ pub enum DataType {
     Binary,
     /// Opaque binary data of fixed size.
     /// Enum parameter specifies the number of bytes per value.
-    FixedSizeBinary(i32),
+    FixedSizeBinary(usize),
     /// Opaque binary data of variable length and 64-bit offsets.
     LargeBinary,
     /// A variable-length string in Unicode with UTF-8 encoding.
@@ -84,14 +84,14 @@ pub enum DataType {
     /// A list of some logical data type with variable length.
     List(Box<Field>),
     /// A list of some logical data type with fixed length.
-    FixedSizeList(Box<Field>, i32),
+    FixedSizeList(Box<Field>, usize),
     /// A list of some logical data type with variable length and 64-bit offsets.
     LargeList(Box<Field>),
     /// A nested datatype that contains a number of sub-fields.
     Struct(Vec<Field>),
     /// A nested datatype that can represent slots of differing types.
-    /// Third argument represents sparsness
-    Union(Vec<Field>, Option<Vec<i32>>, bool),
+    /// Third argument represents mode
+    Union(Vec<Field>, Option<Vec<i32>>, UnionMode),
     /// A nested type that is represented as
     ///
     /// List<entries: Struct<key: K, value: V>>
@@ -128,7 +128,7 @@ pub enum DataType {
     ///
     /// This type mostly used to represent low cardinality string
     /// arrays or a limited set of primitive types as integers.
-    Dictionary(Box<DataType>, Box<DataType>),
+    Dictionary(IntegerType, Box<DataType>),
     /// Decimal value with precision and scale
     /// precision is the number of digits in the number and
     /// scale is the number of decimal places.
@@ -144,6 +144,37 @@ impl std::fmt::Display for DataType {
     }
 }
 
+/// Mode of [`DataType::Union`]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum UnionMode {
+    /// Dense union
+    Dense,
+    /// Sparse union
+    Sparse,
+}
+
+impl UnionMode {
+    /// Constructs a [`UnionMode::Sparse`] if the input bool is true,
+    /// or otherwise constructs a [`UnionMode::Dense`]
+    pub fn sparse(is_sparse: bool) -> Self {
+        if is_sparse {
+            Self::Sparse
+        } else {
+            Self::Dense
+        }
+    }
+
+    /// Returns whether the mode is sparse
+    pub fn is_sparse(&self) -> bool {
+        matches!(self, Self::Sparse)
+    }
+
+    /// Returns whether the mode is dense
+    pub fn is_dense(&self) -> bool {
+        matches!(self, Self::Dense)
+    }
+}
+
 /// The time units defined in Arrow.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum TimeUnit {
@@ -236,7 +267,7 @@ impl DataType {
             Struct(_) => PhysicalType::Struct,
             Union(_, _, _) => PhysicalType::Union,
             Map(_, _) => PhysicalType::Map,
-            Dictionary(key, _) => PhysicalType::Dictionary(to_dictionary_index_type(key.as_ref())),
+            Dictionary(key, _) => PhysicalType::Dictionary(*key),
             Extension(_, key, _) => key.to_physical_type(),
         }
     }
@@ -253,17 +284,18 @@ impl DataType {
     }
 }
 
-fn to_dictionary_index_type(data_type: &DataType) -> DictionaryIndexType {
-    match data_type {
-        DataType::Int8 => DictionaryIndexType::Int8,
-        DataType::Int16 => DictionaryIndexType::Int16,
-        DataType::Int32 => DictionaryIndexType::Int32,
-        DataType::Int64 => DictionaryIndexType::Int64,
-        DataType::UInt8 => DictionaryIndexType::UInt8,
-        DataType::UInt16 => DictionaryIndexType::UInt16,
-        DataType::UInt32 => DictionaryIndexType::UInt32,
-        DataType::UInt64 => DictionaryIndexType::UInt64,
-        _ => ::core::unreachable!("A dictionary key type can only be of integer types"),
+impl From<IntegerType> for DataType {
+    fn from(item: IntegerType) -> Self {
+        match item {
+            IntegerType::Int8 => DataType::Int8,
+            IntegerType::Int16 => DataType::Int16,
+            IntegerType::Int32 => DataType::Int32,
+            IntegerType::Int64 => DataType::Int64,
+            IntegerType::UInt8 => DataType::UInt8,
+            IntegerType::UInt16 => DataType::UInt16,
+            IntegerType::UInt32 => DataType::UInt32,
+            IntegerType::UInt64 => DataType::UInt64,
+        }
     }
 }
 
