@@ -1,7 +1,9 @@
 use std::{collections::BTreeMap, convert::TryInto, ffi::CStr, ffi::CString, ptr};
 
 use crate::{
-    datatypes::{DataType, Extension, Field, IntervalUnit, Metadata, TimeUnit},
+    datatypes::{
+        DataType, Extension, Field, IntegerType, IntervalUnit, Metadata, TimeUnit, UnionMode,
+    },
     error::{ArrowError, Result},
 };
 
@@ -63,6 +65,9 @@ impl Ffi_ArrowSchema {
             DataType::List(field) => {
                 vec![Box::new(Ffi_ArrowSchema::new(field.as_ref()))]
             }
+            DataType::FixedSizeList(field, _) => {
+                vec![Box::new(Ffi_ArrowSchema::new(field.as_ref()))]
+            }
             DataType::LargeList(field) => {
                 vec![Box::new(Ffi_ArrowSchema::new(field.as_ref()))]
             }
@@ -207,12 +212,9 @@ impl Drop for Ffi_ArrowSchema {
 pub(crate) unsafe fn to_field(schema: &Ffi_ArrowSchema) -> Result<Field> {
     let dictionary = schema.dictionary();
     let data_type = if let Some(dictionary) = dictionary {
-        let indices_data_type = to_data_type(schema)?;
+        let indices = to_integer_type(schema.format())?;
         let values = to_field(dictionary)?;
-        DataType::Dictionary(
-            Box::new(indices_data_type),
-            Box::new(values.data_type().clone()),
-        )
+        DataType::Dictionary(indices, Box::new(values.data_type().clone()))
     } else {
         to_data_type(schema)?
     };
@@ -229,6 +231,25 @@ pub(crate) unsafe fn to_field(schema: &Ffi_ArrowSchema) -> Result<Field> {
     Ok(field)
 }
 
+fn to_integer_type(format: &str) -> Result<IntegerType> {
+    use IntegerType::*;
+    Ok(match format {
+        "c" => Int8,
+        "C" => UInt8,
+        "s" => Int16,
+        "S" => UInt16,
+        "i" => Int32,
+        "I" => UInt32,
+        "l" => Int64,
+        "L" => UInt64,
+        _ => {
+            return Err(ArrowError::OutOfSpec(
+                "Dictionary indices can only be integers".to_string(),
+            ))
+        }
+    })
+}
+
 unsafe fn to_data_type(schema: &Ffi_ArrowSchema) -> Result<DataType> {
     Ok(match schema.format() {
         "n" => DataType::Null,
@@ -290,45 +311,62 @@ unsafe fn to_data_type(schema: &Ffi_ArrowSchema) -> Result<DataType> {
                 DataType::Timestamp(TimeUnit::Microsecond, Some(parts[1].to_string()))
             } else if parts.len() == 2 && parts[0] == "tsn" {
                 DataType::Timestamp(TimeUnit::Nanosecond, Some(parts[1].to_string()))
+            } else if parts.len() == 2 && parts[0] == "w" {
+                let size = parts[1].parse::<usize>().map_err(|_| {
+                    ArrowError::OutOfSpec("size is not a valid integer".to_string())
+                })?;
+                DataType::FixedSizeBinary(size)
+            } else if parts.len() == 2 && parts[0] == "+w" {
+                let size = parts[1].parse::<usize>().map_err(|_| {
+                    ArrowError::OutOfSpec("size is not a valid integer".to_string())
+                })?;
+                let child = to_field(schema.child(0))?;
+                DataType::FixedSizeList(Box::new(child), size)
             } else if parts.len() == 2 && parts[0] == "d" {
                 let parts = parts[1].split(',').collect::<Vec<_>>();
                 if parts.len() < 2 || parts.len() > 3 {
-                    return Err(ArrowError::Ffi(
+                    return Err(ArrowError::OutOfSpec(
                         "Decimal must contain 2 or 3 comma-separated values".to_string(),
                     ));
                 };
                 if parts.len() == 3 {
                     let bit_width = parts[0].parse::<usize>().map_err(|_| {
-                        ArrowError::Ffi("Decimal bit width is not a valid integer".to_string())
+                        ArrowError::OutOfSpec(
+                            "Decimal bit width is not a valid integer".to_string(),
+                        )
                     })?;
                     if bit_width != 128 {
-                        return Err(ArrowError::Ffi("Decimal256 is not supported".to_string()));
+                        return Err(ArrowError::OutOfSpec(
+                            "Decimal256 is not supported".to_string(),
+                        ));
                     }
                 }
                 let precision = parts[0].parse::<usize>().map_err(|_| {
-                    ArrowError::Ffi("Decimal precision is not a valid integer".to_string())
+                    ArrowError::OutOfSpec("Decimal precision is not a valid integer".to_string())
                 })?;
                 let scale = parts[1].parse::<usize>().map_err(|_| {
-                    ArrowError::Ffi("Decimal scale is not a valid integer".to_string())
+                    ArrowError::OutOfSpec("Decimal scale is not a valid integer".to_string())
                 })?;
                 DataType::Decimal(precision, scale)
             } else if !parts.is_empty() && ((parts[0] == "+us") || (parts[0] == "+ud")) {
                 // union
-                let is_sparse = parts[0] == "+us";
+                let mode = UnionMode::sparse(parts[0] == "+us");
                 let type_ids = parts[1]
                     .split(',')
                     .map(|x| {
                         x.parse::<i32>().map_err(|_| {
-                            ArrowError::Ffi("Union type id is not a valid integer".to_string())
+                            ArrowError::OutOfSpec(
+                                "Union type id is not a valid integer".to_string(),
+                            )
                         })
                     })
                     .collect::<Result<Vec<_>>>()?;
                 let fields = (0..schema.n_children as usize)
                     .map(|x| to_field(schema.child(x)))
                     .collect::<Result<Vec<_>>>()?;
-                DataType::Union(fields, Some(type_ids), is_sparse)
+                DataType::Union(fields, Some(type_ids), mode)
             } else {
-                return Err(ArrowError::Ffi(format!(
+                return Err(ArrowError::OutOfSpec(format!(
                     "The datatype \"{}\" is still not supported in Rust implementation",
                     other
                 )));
@@ -380,10 +418,10 @@ fn to_format(data_type: &DataType) -> String {
         }
         DataType::Timestamp(unit, tz) => {
             let unit = match unit {
-                TimeUnit::Second => "s".to_string(),
-                TimeUnit::Millisecond => "m".to_string(),
-                TimeUnit::Microsecond => "u".to_string(),
-                TimeUnit::Nanosecond => "n".to_string(),
+                TimeUnit::Second => "s",
+                TimeUnit::Millisecond => "m",
+                TimeUnit::Microsecond => "u",
+                TimeUnit::Nanosecond => "n",
             };
             format!(
                 "ts{}:{}",
@@ -395,10 +433,10 @@ fn to_format(data_type: &DataType) -> String {
         DataType::List(_) => "+l".to_string(),
         DataType::LargeList(_) => "+L".to_string(),
         DataType::Struct(_) => "+s".to_string(),
-        DataType::FixedSizeBinary(size) => format!("w{}", size),
+        DataType::FixedSizeBinary(size) => format!("w:{}", size),
         DataType::FixedSizeList(_, size) => format!("+w:{}", size),
-        DataType::Union(f, ids, is_sparse) => {
-            let sparsness = if *is_sparse { 's' } else { 'd' };
+        DataType::Union(f, ids, mode) => {
+            let sparsness = if mode.is_sparse() { 's' } else { 'd' };
             let mut r = format!("+u{}:", sparsness);
             let ids = if let Some(ids) = ids {
                 ids.iter()
@@ -411,7 +449,7 @@ fn to_format(data_type: &DataType) -> String {
             r
         }
         DataType::Map(_, _) => "+m".to_string(),
-        DataType::Dictionary(index, _) => to_format(index.as_ref()),
+        DataType::Dictionary(index, _) => to_format(&(*index).into()),
         DataType::Extension(_, inner, _) => to_format(inner.as_ref()),
     }
 }
@@ -419,11 +457,12 @@ fn to_format(data_type: &DataType) -> String {
 pub(super) fn get_field_child(field: &Field, index: usize) -> Result<Field> {
     match (index, field.data_type()) {
         (0, DataType::List(field)) => Ok(field.as_ref().clone()),
+        (0, DataType::FixedSizeList(field, _)) => Ok(field.as_ref().clone()),
         (0, DataType::LargeList(field)) => Ok(field.as_ref().clone()),
         (0, DataType::Map(field, _)) => Ok(field.as_ref().clone()),
         (index, DataType::Struct(fields)) => Ok(fields[index].clone()),
         (index, DataType::Union(fields, _, _)) => Ok(fields[index].clone()),
-        (child, data_type) => Err(ArrowError::Ffi(format!(
+        (child, data_type) => Err(ArrowError::OutOfSpec(format!(
             "Requested child {} to type {:?} that has no such child",
             child, data_type
         ))),
