@@ -28,7 +28,8 @@ mod ipc {
 }
 
 use crate::datatypes::{
-    get_extension, DataType, Extension, Field, IntervalUnit, Metadata, Schema, TimeUnit,
+    get_extension, DataType, Extension, Field, IntegerType, IntervalUnit, Metadata, Schema,
+    TimeUnit, UnionMode,
 };
 use crate::io::ipc::endianess::is_native_little_endian;
 
@@ -142,18 +143,18 @@ fn get_data_type(field: ipc::Field, extension: Extension, may_be_dictionary: boo
         if may_be_dictionary {
             let int = dictionary.indexType().unwrap();
             let index_type = match (int.bitWidth(), int.is_signed()) {
-                (8, true) => DataType::Int8,
-                (8, false) => DataType::UInt8,
-                (16, true) => DataType::Int16,
-                (16, false) => DataType::UInt16,
-                (32, true) => DataType::Int32,
-                (32, false) => DataType::UInt32,
-                (64, true) => DataType::Int64,
-                (64, false) => DataType::UInt64,
+                (8, true) => IntegerType::Int8,
+                (8, false) => IntegerType::UInt8,
+                (16, true) => IntegerType::Int16,
+                (16, false) => IntegerType::UInt16,
+                (32, true) => IntegerType::Int32,
+                (32, false) => IntegerType::UInt32,
+                (64, true) => IntegerType::Int64,
+                (64, false) => IntegerType::UInt64,
                 _ => panic!("Unexpected bitwidth and signed"),
             };
             return DataType::Dictionary(
-                Box::new(index_type),
+                index_type,
                 Box::new(get_data_type(field, extension, false)),
             );
         }
@@ -191,7 +192,7 @@ fn get_data_type(field: ipc::Field, extension: Extension, may_be_dictionary: boo
         ipc::Type::LargeUtf8 => DataType::LargeUtf8,
         ipc::Type::FixedSizeBinary => {
             let fsb = field.type_as_fixed_size_binary().unwrap();
-            DataType::FixedSizeBinary(fsb.byteWidth())
+            DataType::FixedSizeBinary(fsb.byteWidth() as usize)
         }
         ipc::Type::FloatingPoint => {
             let float = field.type_as_floating_point().unwrap();
@@ -273,7 +274,7 @@ fn get_data_type(field: ipc::Field, extension: Extension, may_be_dictionary: boo
                 panic!("expect a list to have one child")
             }
             let fsl = field.type_as_fixed_size_list().unwrap();
-            DataType::FixedSizeList(Box::new(children.get(0).into()), fsl.listSize())
+            DataType::FixedSizeList(Box::new(children.get(0).into()), fsl.listSize() as usize)
         }
         ipc::Type::Struct_ => {
             let mut fields = vec![];
@@ -292,7 +293,7 @@ fn get_data_type(field: ipc::Field, extension: Extension, may_be_dictionary: boo
         ipc::Type::Union => {
             let type_ = field.type_as_union().unwrap();
 
-            let is_sparse = type_.mode() == ipc::UnionMode::Sparse;
+            let mode = UnionMode::sparse(type_.mode() == ipc::UnionMode::Sparse);
 
             let ids = type_.typeIds().map(|x| x.iter().collect());
 
@@ -303,7 +304,7 @@ fn get_data_type(field: ipc::Field, extension: Extension, may_be_dictionary: boo
             } else {
                 vec![]
             };
-            DataType::Union(fields, ids, is_sparse)
+            DataType::Union(fields, ids, mode)
         }
         ipc::Type::Map => {
             let map = field.type_as_map().unwrap();
@@ -662,24 +663,8 @@ pub(crate) fn get_fb_field_type<'a>(
             }
         }
         Struct(fields) => {
-            // struct's fields are children
-            let mut children = vec![];
-            for field in fields {
-                let inner_types = get_fb_field_type(field.data_type(), field.is_nullable(), fbb);
-                let field_name = fbb.create_string(field.name());
-                children.push(ipc::Field::create(
-                    fbb,
-                    &ipc::FieldArgs {
-                        name: Some(field_name),
-                        nullable: field.is_nullable(),
-                        type_type: inner_types.type_type,
-                        type_: Some(inner_types.type_),
-                        dictionary: None,
-                        children: inner_types.children,
-                        custom_metadata: None,
-                    },
-                ));
-            }
+            let children: Vec<_> = fields.iter().map(|field| build_field(fbb, field)).collect();
+
             FbFieldType {
                 type_type,
                 type_: ipc::Struct_Builder::new(fbb).finish().as_union_value(),
@@ -704,13 +689,13 @@ pub(crate) fn get_fb_field_type<'a>(
                 children: Some(fbb.create_vector(&empty_fields[..])),
             }
         }
-        Union(fields, ids, is_sparse) => {
+        Union(fields, ids, mode) => {
             let children: Vec<_> = fields.iter().map(|field| build_field(fbb, field)).collect();
 
             let ids = ids.as_ref().map(|ids| fbb.create_vector(ids));
 
             let mut builder = ipc::UnionBuilder::new(fbb);
-            builder.add_mode(if *is_sparse {
+            builder.add_mode(if mode.is_sparse() {
                 ipc::UnionMode::Sparse
             } else {
                 ipc::UnionMode::Dense
@@ -740,28 +725,26 @@ pub(crate) fn get_fb_field_type<'a>(
 
 /// Create an IPC dictionary encoding
 pub(crate) fn get_fb_dictionary<'a>(
-    index_type: &DataType,
+    index_type: &IntegerType,
     dict_id: i64,
     dict_is_ordered: bool,
     fbb: &mut FlatBufferBuilder<'a>,
 ) -> WIPOffset<ipc::DictionaryEncoding<'a>> {
-    use DataType::*;
+    use IntegerType::*;
     // We assume that the dictionary index type (as an integer) has already been
     // validated elsewhere, and can safely assume we are dealing with integers
     let mut index_builder = ipc::IntBuilder::new(fbb);
 
-    match *index_type {
+    match index_type {
         Int8 | Int16 | Int32 | Int64 => index_builder.add_is_signed(true),
         UInt8 | UInt16 | UInt32 | UInt64 => index_builder.add_is_signed(false),
-        _ => {}
     }
 
-    match *index_type {
+    match index_type {
         Int8 | UInt8 => index_builder.add_bitWidth(8),
         Int16 | UInt16 => index_builder.add_bitWidth(16),
         Int32 | UInt32 => index_builder.add_bitWidth(32),
         Int64 | UInt64 => index_builder.add_bitWidth(64),
-        _ => {}
     }
 
     let index_builder = index_builder.finish();
@@ -908,14 +891,14 @@ mod tests {
                 Field::new("struct<>", DataType::Struct(vec![]), true),
                 Field::new_dict(
                     "dictionary<int32, utf8>",
-                    DataType::Dictionary(Box::new(DataType::Int32), Box::new(DataType::Utf8)),
+                    DataType::Dictionary(IntegerType::Int32, Box::new(DataType::Utf8)),
                     true,
                     123,
                     true,
                 ),
                 Field::new_dict(
                     "dictionary<uint8, uint32>",
-                    DataType::Dictionary(Box::new(DataType::UInt8), Box::new(DataType::UInt32)),
+                    DataType::Dictionary(IntegerType::UInt8, Box::new(DataType::UInt32)),
                     true,
                     123,
                     true,
