@@ -19,7 +19,7 @@ pub fn read_fixed_size_binary<R: Read + Seek>(
     is_little_endian: bool,
     compression: Option<ipc::Message::BodyCompression>,
 ) -> Result<FixedSizeBinaryArray> {
-    let field_node = field_nodes.pop_front().unwrap().0;
+    let field_node = field_nodes.pop_front().unwrap();
 
     let validity = read_validity(
         buffers,
@@ -30,8 +30,7 @@ pub fn read_fixed_size_binary<R: Read + Seek>(
         compression,
     )?;
 
-    let length =
-        field_node.length() as usize * (*FixedSizeBinaryArray::get_size(&data_type) as usize);
+    let length = field_node.length() as usize * FixedSizeBinaryArray::get_size(&data_type);
     let values = read_buffer(
         buffers,
         length,
