@@ -1,9 +1,10 @@
-use std::collections::VecDeque;
+use std::collections::{HashMap, VecDeque};
 use std::io::{Read, Seek};
+use std::sync::Arc;
 
 use arrow_format::ipc;
 
-use crate::array::FixedSizeListArray;
+use crate::array::{Array, FixedSizeListArray};
 use crate::datatypes::DataType;
 use crate::error::Result;
 
@@ -16,12 +17,13 @@ pub fn read_fixed_size_list<R: Read + Seek>(
     data_type: DataType,
     buffers: &mut VecDeque<&ipc::Schema::Buffer>,
     reader: &mut R,
+    dictionaries: &HashMap<usize, Arc<dyn Array>>,
     block_offset: u64,
     is_little_endian: bool,
     compression: Option<ipc::Message::BodyCompression>,
     version: ipc::Schema::MetadataVersion,
 ) -> Result<FixedSizeListArray> {
-    let field_node = field_nodes.pop_front().unwrap().0;
+    let field_node = field_nodes.pop_front().unwrap();
 
     let validity = read_validity(
         buffers,
@@ -32,13 +34,14 @@ pub fn read_fixed_size_list<R: Read + Seek>(
         compression,
     )?;
 
-    let (value_data_type, _) = FixedSizeListArray::get_child_and_size(&data_type);
+    let (field, _) = FixedSizeListArray::get_child_and_size(&data_type);
 
     let values = read(
         field_nodes,
-        value_data_type.data_type().clone(),
+        field,
         buffers,
         reader,
+        dictionaries,
         block_offset,
         is_little_endian,
         compression,
