@@ -1,10 +1,11 @@
-use std::collections::VecDeque;
+use std::collections::{HashMap, VecDeque};
 use std::convert::TryInto;
 use std::io::{Read, Seek};
+use std::sync::Arc;
 
 use arrow_format::ipc;
 
-use crate::array::{ListArray, Offset};
+use crate::array::{Array, ListArray, Offset};
 use crate::buffer::Buffer;
 use crate::datatypes::DataType;
 use crate::error::Result;
@@ -18,6 +19,7 @@ pub fn read_list<O: Offset, R: Read + Seek>(
     data_type: DataType,
     buffers: &mut VecDeque<&ipc::Schema::Buffer>,
     reader: &mut R,
+    dictionaries: &HashMap<usize, Arc<dyn Array>>,
     block_offset: u64,
     is_little_endian: bool,
     compression: Option<ipc::Message::BodyCompression>,
@@ -26,7 +28,7 @@ pub fn read_list<O: Offset, R: Read + Seek>(
 where
     Vec<u8>: TryInto<O::Bytes>,
 {
-    let field_node = field_nodes.pop_front().unwrap().0;
+    let field_node = field_nodes.pop_front().unwrap();
 
     let validity = read_validity(
         buffers,
@@ -48,13 +50,14 @@ where
     // Older versions of the IPC format sometimes do not report an offset
     .or_else(|_| Result::Ok(Buffer::<O>::from(&[O::default()])))?;
 
-    let value_data_type = ListArray::<O>::get_child_type(&data_type).clone();
+    let field = ListArray::<O>::get_child_field(&data_type);
 
     let values = read(
         field_nodes,
-        value_data_type,
+        field,
         buffers,
         reader,
+        dictionaries,
         block_offset,
         is_little_endian,
         compression,
