@@ -98,26 +98,20 @@ pub fn read_record_batch<R: Read + Seek>(
     schema: Arc<Schema>,
     projection: Option<(&[usize], Arc<Schema>)>,
     is_little_endian: bool,
-    dictionaries: &[Option<ArrayRef>],
+    dictionaries: &HashMap<usize, Arc<dyn Array>>,
     version: MetadataVersion,
     reader: &mut R,
     block_offset: u64,
 ) -> Result<RecordBatch> {
-    let buffers = batch
-        .buffers()
-        .ok_or_else(|| ArrowError::Ipc("Unable to get buffers from IPC RecordBatch".to_string()))?;
+    let buffers = batch.buffers().ok_or_else(|| {
+        ArrowError::OutOfSpec("Unable to get buffers from IPC RecordBatch".to_string())
+    })?;
     let mut buffers: VecDeque<&ipc::Schema::Buffer> = buffers.iter().collect();
     let field_nodes = batch.nodes().ok_or_else(|| {
-        ArrowError::Ipc("Unable to get field nodes from IPC RecordBatch".to_string())
+        ArrowError::OutOfSpec("Unable to get field nodes from IPC RecordBatch".to_string())
     })?;
 
-    // This is a bug fix: we should have one dictionary per node, not schema field
-    let dictionaries = dictionaries.iter().chain(std::iter::repeat(&None));
-
-    let mut field_nodes = field_nodes
-        .iter()
-        .zip(dictionaries)
-        .collect::<VecDeque<_>>();
+    let mut field_nodes = field_nodes.iter().collect::<VecDeque<_>>();
 
     let (schema, columns) = if let Some(projection) = projection {
         let projected_schema = projection.1.clone();
@@ -128,9 +122,10 @@ pub fn read_record_batch<R: Read + Seek>(
             .map(|maybe_field| match maybe_field {
                 ProjectionResult::Selected(field) => Some(read(
                     &mut field_nodes,
-                    field.data_type().clone(),
+                    field,
                     &mut buffers,
                     reader,
+                    dictionaries,
                     block_offset,
                     is_little_endian,
                     batch.compression(),
@@ -151,9 +146,10 @@ pub fn read_record_batch<R: Read + Seek>(
             .map(|field| {
                 read(
                     &mut field_nodes,
-                    field.data_type().clone(),
+                    field,
                     &mut buffers,
                     reader,
+                    dictionaries,
                     block_offset,
                     is_little_endian,
                     batch.compression(),
@@ -166,13 +162,62 @@ pub fn read_record_batch<R: Read + Seek>(
     RecordBatch::try_new(schema, columns)
 }
 
+fn find_first_dict_field_d(id: usize, data_type: &DataType) -> Option<&Field> {
+    use DataType::*;
+    match data_type {
+        Dictionary(_, inner) => find_first_dict_field_d(id, inner.as_ref()),
+        Map(field, _) => find_first_dict_field(id, field.as_ref()),
+        List(field) => find_first_dict_field(id, field.as_ref()),
+        LargeList(field) => find_first_dict_field(id, field.as_ref()),
+        FixedSizeList(field, _) => find_first_dict_field(id, field.as_ref()),
+        Union(fields, _, _) => {
+            for field in fields {
+                if let Some(f) = find_first_dict_field(id, field) {
+                    return Some(f);
+                }
+            }
+            None
+        }
+        Struct(fields) => {
+            for field in fields {
+                if let Some(f) = find_first_dict_field(id, field) {
+                    return Some(f);
+                }
+            }
+            None
+        }
+        _ => None,
+    }
+}
+
+fn find_first_dict_field(id: usize, field: &Field) -> Option<&Field> {
+    if let DataType::Dictionary(_, _) = &field.data_type {
+        if field.dict_id as usize == id {
+            return Some(field);
+        }
+    }
+    find_first_dict_field_d(id, &field.data_type)
+}
+
+fn first_dict_field(id: usize, fields: &[Field]) -> Result<&Field> {
+    for field in fields {
+        if let Some(field) = find_first_dict_field(id, field) {
+            return Ok(field);
+        }
+    }
+    Err(ArrowError::OutOfSpec(format!(
+        "dictionary id {} not found in schema",
+        id
+    )))
+}
+
 /// Read the dictionary from the buffer and provided metadata,
-/// updating the `dictionaries_by_field` with the resulting dictionary
+/// updating the `dictionaries` with the resulting dictionary
 pub fn read_dictionary<R: Read + Seek>(
     batch: ipc::Message::DictionaryBatch,
     schema: &Schema,
     is_little_endian: bool,
-    dictionaries_by_field: &mut [Option<ArrayRef>],
+    dictionaries: &mut HashMap<usize, Arc<dyn Array>>,
     reader: &mut R,
     block_offset: u64,
 ) -> Result<()> {
@@ -183,10 +228,7 @@ pub fn read_dictionary<R: Read + Seek>(
     }
 
     let id = batch.id();
-    let fields_using_this_dictionary = schema.fields_with_dict_id(id);
-    let first_field = fields_using_this_dictionary.first().ok_or_else(|| {
-        ArrowError::InvalidArgumentError("dictionary id not found in schema".to_string())
-    })?;
+    let first_field = first_dict_field(id as usize, &schema.fields)?;
 
     // As the dictionary batch does not contain the type of the
     // values array, we need to retrieve this from the schema.
@@ -204,7 +246,7 @@ pub fn read_dictionary<R: Read + Seek>(
                 schema,
                 None,
                 is_little_endian,
-                dictionaries_by_field,
+                dictionaries,
                 MetadataVersion::V5,
                 reader,
                 block_offset,
@@ -217,16 +259,7 @@ pub fn read_dictionary<R: Read + Seek>(
         ArrowError::InvalidArgumentError("dictionary id not found in schema".to_string())
     })?;
 
-    // for all fields with this dictionary id, update the dictionaries vector
-    // in the reader. Note that a dictionary batch may be shared between many fields.
-    // We don't currently record the isOrdered field. This could be general
-    // attributes of arrays.
-    for (i, field) in schema.fields().iter().enumerate() {
-        if field.dict_id() == Some(id) {
-            // Add (possibly multiple) array refs to the dictionaries array.
-            dictionaries_by_field[i] = Some(dictionary_values.clone());
-        }
-    }
+    dictionaries.insert(id as usize, dictionary_values);
 
     Ok(())
 }
