@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::collections::HashMap;
 use std::io::Read;
 use std::sync::Arc;
 
@@ -30,8 +31,6 @@ use super::super::convert;
 use super::super::CONTINUATION_MARKER;
 use super::common::*;
 
-type ArrayRef = Arc<dyn Array>;
-
 #[derive(Debug)]
 pub struct StreamMetadata {
     /// The schema that is read from the stream's first message
@@ -60,13 +59,14 @@ pub fn read_stream_metadata<R: Read>(reader: &mut R) -> Result<StreamMetadata> {
     let mut meta_buffer = vec![0; meta_len as usize];
     reader.read_exact(&mut meta_buffer)?;
 
-    let message = ipc::Message::root_as_message(meta_buffer.as_slice())
-        .map_err(|err| ArrowError::Ipc(format!("Unable to get root as message: {:?}", err)))?;
+    let message = ipc::Message::root_as_message(meta_buffer.as_slice()).map_err(|err| {
+        ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
+    })?;
     let version = message.version();
     // message header is a Schema, so read it
     let ipc_schema: ipc::Schema::Schema = message
         .header_as_schema()
-        .ok_or_else(|| ArrowError::Ipc("Unable to read IPC message as schema".to_string()))?;
+        .ok_or_else(|| ArrowError::OutOfSpec("Unable to read IPC message as schema".to_string()))?;
     let (schema, is_little_endian) = convert::fb_to_schema(ipc_schema);
     let schema = Arc::new(schema);
 
@@ -110,15 +110,17 @@ impl StreamState {
 
 /// Reads the next item, yielding `None` if the stream is done,
 /// and a [`StreamState`] otherwise.
-pub fn read_next<R: Read>(
+fn read_next<R: Read>(
     reader: &mut R,
     metadata: &StreamMetadata,
-    dictionaries_by_field: &mut Vec<Option<ArrayRef>>,
+    dictionaries: &mut HashMap<usize, Arc<dyn Array>>,
+    message_buffer: &mut Vec<u8>,
+    data_buffer: &mut Vec<u8>,
 ) -> Result<Option<StreamState>> {
     // determine metadata length
-    let mut meta_size: [u8; 4] = [0; 4];
+    let mut meta_length: [u8; 4] = [0; 4];
 
-    match reader.read_exact(&mut meta_size) {
+    match reader.read_exact(&mut meta_length) {
         Ok(()) => (),
         Err(e) => {
             return if e.kind() == std::io::ErrorKind::UnexpectedEof {
@@ -132,47 +134,49 @@ pub fn read_next<R: Read>(
         }
     }
 
-    let meta_len = {
+    let meta_length = {
         // If a continuation marker is encountered, skip over it and read
         // the size from the next four bytes.
-        if meta_size == CONTINUATION_MARKER {
-            reader.read_exact(&mut meta_size)?;
+        if meta_length == CONTINUATION_MARKER {
+            reader.read_exact(&mut meta_length)?;
         }
-        i32::from_le_bytes(meta_size)
+        i32::from_le_bytes(meta_length) as usize
     };
 
-    if meta_len == 0 {
+    if meta_length == 0 {
         // the stream has ended, mark the reader as finished
         return Ok(None);
     }
 
-    let mut meta_buffer = vec![0; meta_len as usize];
-    reader.read_exact(&mut meta_buffer)?;
+    message_buffer.clear();
+    message_buffer.resize(meta_length, 0);
+    reader.read_exact(message_buffer)?;
 
-    let vecs = &meta_buffer.to_vec();
-    let message = ipc::Message::root_as_message(vecs)
-        .map_err(|err| ArrowError::Ipc(format!("Unable to get root as message: {:?}", err)))?;
+    let message = ipc::Message::root_as_message(message_buffer).map_err(|err| {
+        ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
+    })?;
 
     match message.header_type() {
-        ipc::Message::MessageHeader::Schema => Err(ArrowError::Ipc(
+        ipc::Message::MessageHeader::Schema => Err(ArrowError::OutOfSpec(
             "Not expecting a schema when messages are read".to_string(),
         )),
         ipc::Message::MessageHeader::RecordBatch => {
             let batch = message.header_as_record_batch().ok_or_else(|| {
-                ArrowError::Ipc("Unable to read IPC message as record batch".to_string())
+                ArrowError::OutOfSpec("Unable to read IPC message as record batch".to_string())
             })?;
             // read the block that makes up the record batch into a buffer
-            let mut buf = vec![0; message.bodyLength() as usize];
-            reader.read_exact(&mut buf)?;
+            data_buffer.clear();
+            data_buffer.resize(message.bodyLength() as usize, 0);
+            reader.read_exact(data_buffer)?;
 
-            let mut reader = std::io::Cursor::new(buf);
+            let mut reader = std::io::Cursor::new(data_buffer);
 
             read_record_batch(
                 batch,
                 metadata.schema.clone(),
                 None,
                 metadata.is_little_endian,
-                dictionaries_by_field,
+                dictionaries,
                 metadata.version,
                 &mut reader,
                 0,
@@ -181,7 +185,7 @@ pub fn read_next<R: Read>(
         }
         ipc::Message::MessageHeader::DictionaryBatch => {
             let batch = message.header_as_dictionary_batch().ok_or_else(|| {
-                ArrowError::Ipc("Unable to read IPC message as dictionary batch".to_string())
+                ArrowError::OutOfSpec("Unable to read IPC message as dictionary batch".to_string())
             })?;
             // read the block that makes up the dictionary batch into a buffer
             let mut buf = vec![0; message.bodyLength() as usize];
@@ -193,16 +197,16 @@ pub fn read_next<R: Read>(
                 batch,
                 &metadata.schema,
                 metadata.is_little_endian,
-                dictionaries_by_field,
+                dictionaries,
                 &mut dict_reader,
                 0,
             )?;
 
             // read the next message until we encounter a RecordBatch
-            read_next(reader, metadata, dictionaries_by_field)
+            read_next(reader, metadata, dictionaries, message_buffer, data_buffer)
         }
         ipc::Message::MessageHeader::NONE => Ok(Some(StreamState::Waiting)),
-        t => Err(ArrowError::Ipc(format!(
+        t => Err(ArrowError::OutOfSpec(format!(
             "Reading types other than record batches not yet supported, unable to read {:?} ",
             t
         ))),
@@ -218,8 +222,10 @@ pub fn read_next<R: Read>(
 pub struct StreamReader<R: Read> {
     reader: R,
     metadata: StreamMetadata,
-    dictionaries_by_field: Vec<Option<ArrayRef>>,
+    dictionaries: HashMap<usize, Arc<dyn Array>>,
     finished: bool,
+    data_buffer: Vec<u8>,
+    message_buffer: Vec<u8>,
 }
 
 impl<R: Read> StreamReader<R> {
@@ -229,12 +235,13 @@ impl<R: Read> StreamReader<R> {
     /// encounter a schema.
     /// To check if the reader is done, use `is_finished(self)`
     pub fn new(reader: R, metadata: StreamMetadata) -> Self {
-        let fields = metadata.schema.fields().len();
         Self {
             reader,
             metadata,
-            dictionaries_by_field: vec![None; fields],
+            dictionaries: Default::default(),
             finished: false,
+            data_buffer: vec![],
+            message_buffer: vec![],
         }
     }
 
@@ -255,7 +262,9 @@ impl<R: Read> StreamReader<R> {
         let batch = read_next(
             &mut self.reader,
             &self.metadata,
-            &mut self.dictionaries_by_field,
+            &mut self.dictionaries,
+            &mut self.message_buffer,
+            &mut self.data_buffer,
         )?;
         if batch.is_none() {
             self.finished = true;
