@@ -4,10 +4,9 @@ use arrow_format::ipc::flatbuffers::FlatBufferBuilder;
 use crate::datatypes::*;
 
 use super::super::convert;
-use super::MetadataVersion;
 
 /// Converts
-pub fn schema_to_bytes(schema: &Schema, version: MetadataVersion) -> Vec<u8> {
+pub fn schema_to_bytes(schema: &Schema) -> Vec<u8> {
     let mut fbb = FlatBufferBuilder::new();
     let schema = {
         let fb = convert::schema_to_fb_offset(&mut fbb, schema);
@@ -15,7 +14,7 @@ pub fn schema_to_bytes(schema: &Schema, version: MetadataVersion) -> Vec<u8> {
     };
 
     let mut message = ipc::Message::MessageBuilder::new(&mut fbb);
-    message.add_version(version);
+    message.add_version(ipc::Schema::MetadataVersion::V5);
     message.add_header_type(ipc::Message::MessageHeader::Schema);
     message.add_bodyLength(0);
     message.add_header(schema);
