@@ -1,20 +1,3 @@
-// Licensed to the Apache Software Foundation (ASF) under one
-// or more contributor license agreements.  See the NOTICE file
-// distributed with this work for additional information
-// regarding copyright ownership.  The ASF licenses this file
-// to you under the Apache License, Version 2.0 (the
-// "License"); you may not use this file except in compliance
-// with the License.  You may obtain a copy of the License at
-//
-//   http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
 use arrow_format::ipc::{Message, Schema};
 
 use crate::{
@@ -25,8 +8,9 @@ use crate::{
     types::NativeType,
 };
 
+use super::super::compression;
 use super::super::endianess::is_native_little_endian;
-use super::common::pad_to_8;
+use super::common::{pad_to_8, Compression};
 
 fn _write_primitive<T: NativeType>(
     array: &PrimitiveArray<T>,
@@ -34,8 +18,16 @@ fn _write_primitive<T: NativeType>(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
-    write_bitmap(array.validity(), array.len(), buffers, arrow_data, offset);
+    write_bitmap(
+        array.validity(),
+        array.len(),
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
 
     write_buffer(
         array.values(),
@@ -43,6 +35,7 @@ fn _write_primitive<T: NativeType>(
         arrow_data,
         offset,
         is_little_endian,
+        compression,
     )
 }
 
@@ -52,9 +45,17 @@ fn write_primitive<T: NativeType>(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<PrimitiveArray<T>>().unwrap();
-    _write_primitive(array, buffers, arrow_data, offset, is_little_endian);
+    _write_primitive(
+        array,
+        buffers,
+        arrow_data,
+        offset,
+        is_little_endian,
+        compression,
+    );
 }
 
 fn write_boolean(
@@ -63,19 +64,29 @@ fn write_boolean(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     _: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<BooleanArray>().unwrap();
 
-    write_bitmap(array.validity(), array.len(), buffers, arrow_data, offset);
+    write_bitmap(
+        array.validity(),
+        array.len(),
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
     write_bitmap(
         Some(&array.values().clone()),
         array.len(),
         buffers,
         arrow_data,
         offset,
+        compression,
     );
 }
 
+#[allow(clippy::too_many_arguments)]
 fn write_generic_binary<O: Offset>(
     validity: Option<&Bitmap>,
     offsets: &[O],
@@ -84,13 +95,28 @@ fn write_generic_binary<O: Offset>(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
-    write_bitmap(validity, offsets.len() - 1, buffers, arrow_data, offset);
+    write_bitmap(
+        validity,
+        offsets.len() - 1,
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
 
     let first = *offsets.first().unwrap();
     let last = *offsets.last().unwrap();
     if first == O::default() {
-        write_buffer(offsets, buffers, arrow_data, offset, is_little_endian);
+        write_buffer(
+            offsets,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        );
     } else {
         write_buffer_from_iter(
             offsets.iter().map(|x| *x - first),
@@ -98,15 +124,16 @@ fn write_generic_binary<O: Offset>(
             arrow_data,
             offset,
             is_little_endian,
+            compression,
         );
     }
 
-    write_buffer(
+    write_bytes(
         &values[first.to_usize()..last.to_usize()],
         buffers,
         arrow_data,
         offset,
-        is_little_endian,
+        compression,
     );
 }
 
@@ -116,6 +143,7 @@ fn write_binary<O: Offset>(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<BinaryArray<O>>().unwrap();
     write_generic_binary(
@@ -126,6 +154,7 @@ fn write_binary<O: Offset>(
         arrow_data,
         offset,
         is_little_endian,
+        compression,
     );
 }
 
@@ -135,6 +164,7 @@ fn write_utf8<O: Offset>(
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<Utf8Array<O>>().unwrap();
     write_generic_binary(
@@ -145,6 +175,7 @@ fn write_utf8<O: Offset>(
         arrow_data,
         offset,
         is_little_endian,
+        compression,
     );
 }
 
@@ -153,20 +184,22 @@ fn write_fixed_size_binary(
     buffers: &mut Vec<Schema::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
-    is_little_endian: bool,
+    _is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array
         .as_any()
         .downcast_ref::<FixedSizeBinaryArray>()
         .unwrap();
-    write_bitmap(array.validity(), array.len(), buffers, arrow_data, offset);
-    write_buffer(
-        array.values(),
+    write_bitmap(
+        array.validity(),
+        array.len(),
         buffers,
         arrow_data,
         offset,
-        is_little_endian,
+        compression,
     );
+    write_bytes(array.values(), buffers, arrow_data, offset, compression);
 }
 
 fn write_list<O: Offset>(
@@ -176,17 +209,32 @@ fn write_list<O: Offset>(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<ListArray<O>>().unwrap();
     let offsets = array.offsets();
     let validity = array.validity();
 
-    write_bitmap(validity, offsets.len() - 1, buffers, arrow_data, offset);
+    write_bitmap(
+        validity,
+        offsets.len() - 1,
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
 
     let first = *offsets.first().unwrap();
     let last = *offsets.last().unwrap();
     if first == O::default() {
-        write_buffer(offsets, buffers, arrow_data, offset, is_little_endian);
+        write_buffer(
+            offsets,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        );
     } else {
         write_buffer_from_iter(
             offsets.iter().map(|x| *x - first),
@@ -194,6 +242,7 @@ fn write_list<O: Offset>(
             arrow_data,
             offset,
             is_little_endian,
+            compression,
         );
     }
 
@@ -207,6 +256,7 @@ fn write_list<O: Offset>(
         nodes,
         offset,
         is_little_endian,
+        compression,
     );
 }
 
@@ -217,9 +267,17 @@ pub fn write_struct(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<StructArray>().unwrap();
-    write_bitmap(array.validity(), array.len(), buffers, arrow_data, offset);
+    write_bitmap(
+        array.validity(),
+        array.len(),
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
     array.values().iter().for_each(|array| {
         write(
             array.as_ref(),
@@ -228,6 +286,7 @@ pub fn write_struct(
             nodes,
             offset,
             is_little_endian,
+            compression,
         );
     });
 }
@@ -239,13 +298,28 @@ pub fn write_union(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<UnionArray>().unwrap();
 
-    write_buffer(array.types(), buffers, arrow_data, offset, is_little_endian);
+    write_buffer(
+        array.types(),
+        buffers,
+        arrow_data,
+        offset,
+        is_little_endian,
+        compression,
+    );
 
     if let Some(offsets) = array.offsets() {
-        write_buffer(offsets, buffers, arrow_data, offset, is_little_endian);
+        write_buffer(
+            offsets,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        );
     }
     array.fields().iter().for_each(|array| {
         write(
@@ -255,6 +329,7 @@ pub fn write_union(
             nodes,
             offset,
             is_little_endian,
+            compression,
         )
     });
 }
@@ -266,17 +341,32 @@ fn write_map(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<MapArray>().unwrap();
     let offsets = array.offsets();
     let validity = array.validity();
 
-    write_bitmap(validity, offsets.len() - 1, buffers, arrow_data, offset);
+    write_bitmap(
+        validity,
+        offsets.len() - 1,
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
 
     let first = *offsets.first().unwrap();
     let last = *offsets.last().unwrap();
     if first == 0 {
-        write_buffer(offsets, buffers, arrow_data, offset, is_little_endian);
+        write_buffer(
+            offsets,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        );
     } else {
         write_buffer_from_iter(
             offsets.iter().map(|x| *x - first),
@@ -284,6 +374,7 @@ fn write_map(
             arrow_data,
             offset,
             is_little_endian,
+            compression,
         );
     }
 
@@ -297,6 +388,7 @@ fn write_map(
         nodes,
         offset,
         is_little_endian,
+        compression,
     );
 }
 
@@ -307,9 +399,17 @@ fn write_fixed_size_list(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     let array = array.as_any().downcast_ref::<FixedSizeListArray>().unwrap();
-    write_bitmap(array.validity(), array.len(), buffers, arrow_data, offset);
+    write_bitmap(
+        array.validity(),
+        array.len(),
+        buffers,
+        arrow_data,
+        offset,
+        compression,
+    );
     write(
         array.values().as_ref(),
         buffers,
@@ -317,10 +417,12 @@ fn write_fixed_size_list(
         nodes,
         offset,
         is_little_endian,
+        compression,
     );
 }
 
 // use `write_keys` to either write keys or values
+#[allow(clippy::too_many_arguments)]
 pub fn _write_dictionary<K: DictionaryKey>(
     array: &dyn Array,
     buffers: &mut Vec<Schema::Buffer>,
@@ -328,11 +430,19 @@ pub fn _write_dictionary<K: DictionaryKey>(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
     write_keys: bool,
 ) -> usize {
     let array = array.as_any().downcast_ref::<DictionaryArray<K>>().unwrap();
     if write_keys {
-        _write_primitive(array.keys(), buffers, arrow_data, offset, is_little_endian);
+        _write_primitive(
+            array.keys(),
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        );
         array.keys().len()
     } else {
         write(
@@ -342,11 +452,13 @@ pub fn _write_dictionary<K: DictionaryKey>(
             nodes,
             offset,
             is_little_endian,
+            compression,
         );
         array.values().len()
     }
 }
 
+#[allow(clippy::too_many_arguments)]
 pub fn write_dictionary(
     array: &dyn Array,
     buffers: &mut Vec<Schema::Buffer>,
@@ -354,11 +466,12 @@ pub fn write_dictionary(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
     write_keys: bool,
 ) -> usize {
     match array.data_type() {
         DataType::Dictionary(key_type, _) => {
-            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+            match_integer_type!(key_type, |$T| {
                 _write_dictionary::<$T>(
                     array,
                     buffers,
@@ -366,6 +479,7 @@ pub fn write_dictionary(
                     nodes,
                     offset,
                     is_little_endian,
+                    compression,
                     write_keys,
                 )
             })
@@ -381,6 +495,7 @@ pub fn write(
     nodes: &mut Vec<Message::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
     nodes.push(Message::FieldNode::new(
         array.len() as i64,
@@ -389,23 +504,93 @@ pub fn write(
     use PhysicalType::*;
     match array.data_type().to_physical_type() {
         Null => (),
-        Boolean => write_boolean(array, buffers, arrow_data, offset, is_little_endian),
+        Boolean => write_boolean(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
         Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
-            write_primitive::<$T>(array, buffers, arrow_data, offset, is_little_endian)
+            write_primitive::<$T>(array, buffers, arrow_data, offset, is_little_endian, compression)
         }),
-        Binary => write_binary::<i32>(array, buffers, arrow_data, offset, is_little_endian),
-        LargeBinary => write_binary::<i64>(array, buffers, arrow_data, offset, is_little_endian),
-        FixedSizeBinary => {
-            write_fixed_size_binary(array, buffers, arrow_data, offset, is_little_endian)
-        }
-        Utf8 => write_utf8::<i32>(array, buffers, arrow_data, offset, is_little_endian),
-        LargeUtf8 => write_utf8::<i64>(array, buffers, arrow_data, offset, is_little_endian),
-        List => write_list::<i32>(array, buffers, arrow_data, nodes, offset, is_little_endian),
-        LargeList => write_list::<i64>(array, buffers, arrow_data, nodes, offset, is_little_endian),
-        FixedSizeList => {
-            write_fixed_size_list(array, buffers, arrow_data, nodes, offset, is_little_endian)
-        }
-        Struct => write_struct(array, buffers, arrow_data, nodes, offset, is_little_endian),
+        Binary => write_binary::<i32>(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        LargeBinary => write_binary::<i64>(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        FixedSizeBinary => write_fixed_size_binary(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        Utf8 => write_utf8::<i32>(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        LargeUtf8 => write_utf8::<i64>(
+            array,
+            buffers,
+            arrow_data,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        List => write_list::<i32>(
+            array,
+            buffers,
+            arrow_data,
+            nodes,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        LargeList => write_list::<i64>(
+            array,
+            buffers,
+            arrow_data,
+            nodes,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        FixedSizeList => write_fixed_size_list(
+            array,
+            buffers,
+            arrow_data,
+            nodes,
+            offset,
+            is_little_endian,
+            compression,
+        ),
+        Struct => write_struct(
+            array,
+            buffers,
+            arrow_data,
+            nodes,
+            offset,
+            is_little_endian,
+            compression,
+        ),
         Dictionary(_) => {
             write_dictionary(
                 array,
@@ -414,50 +599,68 @@ pub fn write(
                 nodes,
                 offset,
                 is_little_endian,
+                compression,
                 true,
             );
         }
         Union => {
-            write_union(array, buffers, arrow_data, nodes, offset, is_little_endian);
+            write_union(
+                array,
+                buffers,
+                arrow_data,
+                nodes,
+                offset,
+                is_little_endian,
+                compression,
+            );
         }
         Map => {
-            write_map(array, buffers, arrow_data, nodes, offset, is_little_endian);
+            write_map(
+                array,
+                buffers,
+                arrow_data,
+                nodes,
+                offset,
+                is_little_endian,
+                compression,
+            );
         }
     }
 }
 
-/// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
 #[inline]
+fn pad_buffer_to_8(buffer: &mut Vec<u8>, length: usize) {
+    let pad_len = pad_to_8(length);
+    buffer.extend_from_slice(&vec![0u8; pad_len]);
+}
+
+/// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
 fn write_bytes(
     bytes: &[u8],
     buffers: &mut Vec<Schema::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
+    compression: Option<Compression>,
 ) {
-    let len = bytes.len();
-    let pad_len = pad_to_8(len as u32);
-    let total_len: i64 = (len + pad_len) as i64;
-    // assert_eq!(len % 8, 0, "Buffer width not a multiple of 8 bytes");
-    buffers.push(Schema::Buffer::new(*offset, total_len));
-    arrow_data.extend_from_slice(bytes);
-    arrow_data.extend_from_slice(&vec![0u8; pad_len][..]);
-    *offset += total_len;
-}
+    let start = arrow_data.len();
+    if let Some(compression) = compression {
+        arrow_data.extend_from_slice(&(bytes.len() as i64).to_le_bytes());
+        match compression {
+            Compression::LZ4 => {
+                compression::compress_lz4(bytes, arrow_data).unwrap();
+            }
+            Compression::ZSTD => {
+                compression::compress_zstd(bytes, arrow_data).unwrap();
+            }
+        }
+    } else {
+        arrow_data.extend_from_slice(bytes);
+    };
 
-/// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
-fn write_bytes_from_iter<I: TrustedLen<Item = u8>>(
-    bytes: I,
-    buffers: &mut Vec<Schema::Buffer>,
-    arrow_data: &mut Vec<u8>,
-    offset: &mut i64,
-) {
-    let len = bytes.size_hint().0;
-    let pad_len = pad_to_8(len as u32);
-    let total_len: i64 = (len + pad_len) as i64;
-    // assert_eq!(len % 8, 0, "Buffer width not a multiple of 8 bytes");
+    pad_buffer_to_8(arrow_data, arrow_data.len() - start);
+
+    let total_len = (arrow_data.len() - start) as i64;
     buffers.push(Schema::Buffer::new(*offset, total_len));
-    arrow_data.extend(bytes);
-    arrow_data.extend_from_slice(&vec![0u8; pad_len][..]);
     *offset += total_len;
 }
 
@@ -467,6 +670,7 @@ fn write_bitmap(
     buffers: &mut Vec<Schema::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
+    compression: Option<Compression>,
 ) {
     match bitmap {
         Some(bitmap) => {
@@ -476,23 +680,42 @@ fn write_bitmap(
                 // case where we can't slice the bitmap as the offsets are not multiple of 8
                 let bytes = Bitmap::from_trusted_len_iter(bitmap.iter());
                 let (slice, _, _) = bytes.as_slice();
-                write_bytes(slice, buffers, arrow_data, offset)
+                write_bytes(slice, buffers, arrow_data, offset, compression)
             } else {
-                write_bytes(slice, buffers, arrow_data, offset)
+                write_bytes(slice, buffers, arrow_data, offset, compression)
             }
         }
         None => {
             // in IPC, the null bitmap is always be present
-            write_bytes_from_iter(
-                std::iter::repeat(1).take(length.saturating_add(7) / 8),
-                buffers,
-                arrow_data,
-                offset,
-            )
+            let slice = vec![0u8; length.saturating_add(7) / 8];
+            write_bytes(&slice, buffers, arrow_data, offset, compression)
         }
     }
 }
 
+/// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
+fn write_buffer<T: NativeType>(
+    buffer: &[T],
+    buffers: &mut Vec<Schema::Buffer>,
+    arrow_data: &mut Vec<u8>,
+    offset: &mut i64,
+    is_little_endian: bool,
+    compression: Option<Compression>,
+) {
+    let start = arrow_data.len();
+    if let Some(compression) = compression {
+        _write_compressed_buffer(buffer, arrow_data, is_little_endian, compression);
+    } else {
+        _write_buffer(buffer, arrow_data, is_little_endian);
+    };
+
+    pad_buffer_to_8(arrow_data, arrow_data.len() - start);
+
+    let total_len = (arrow_data.len() - start) as i64;
+    buffers.push(Schema::Buffer::new(*offset, total_len));
+    *offset += total_len;
+}
+
 #[inline]
 fn _write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
     buffer: I,
@@ -512,8 +735,38 @@ fn _write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
     }
 }
 
+#[inline]
+fn _write_compressed_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
+    buffer: I,
+    arrow_data: &mut Vec<u8>,
+    is_little_endian: bool,
+    compression: Compression,
+) {
+    let len = buffer.size_hint().0;
+    let mut swapped = Vec::with_capacity(len * std::mem::size_of::<T>());
+    if is_little_endian {
+        buffer
+            .map(|x| T::to_le_bytes(&x))
+            .for_each(|x| swapped.extend_from_slice(x.as_ref()));
+    } else {
+        buffer
+            .map(|x| T::to_be_bytes(&x))
+            .for_each(|x| swapped.extend_from_slice(x.as_ref()))
+    };
+    arrow_data.extend_from_slice(&(swapped.len() as i64).to_le_bytes());
+    match compression {
+        Compression::LZ4 => {
+            compression::compress_lz4(&swapped, arrow_data).unwrap();
+        }
+        Compression::ZSTD => {
+            compression::compress_zstd(&swapped, arrow_data).unwrap();
+        }
+    }
+}
+
 fn _write_buffer<T: NativeType>(buffer: &[T], arrow_data: &mut Vec<u8>, is_little_endian: bool) {
     if is_little_endian == is_native_little_endian() {
+        // in native endianess we can use the bytes directly.
         let buffer = unsafe {
             std::slice::from_raw_parts(
                 buffer.as_ptr() as *const u8,
@@ -526,42 +779,54 @@ fn _write_buffer<T: NativeType>(buffer: &[T], arrow_data: &mut Vec<u8>, is_littl
     }
 }
 
-/// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
-fn write_buffer<T: NativeType>(
+fn _write_compressed_buffer<T: NativeType>(
     buffer: &[T],
-    buffers: &mut Vec<Schema::Buffer>,
     arrow_data: &mut Vec<u8>,
-    offset: &mut i64,
     is_little_endian: bool,
+    compression: Compression,
 ) {
-    let len = buffer.len() * std::mem::size_of::<T>();
-    let pad_len = pad_to_8(len as u32);
-    let total_len: i64 = (len + pad_len) as i64;
-    // assert_eq!(len % 8, 0, "Buffer width not a multiple of 8 bytes");
-    buffers.push(Schema::Buffer::new(*offset, total_len));
-
-    _write_buffer(buffer, arrow_data, is_little_endian);
-
-    arrow_data.extend_from_slice(&vec![0u8; pad_len][..]);
-    *offset += total_len;
+    if is_little_endian == is_native_little_endian() {
+        let bytes = unsafe {
+            std::slice::from_raw_parts(
+                buffer.as_ptr() as *const u8,
+                buffer.len() * std::mem::size_of::<T>(),
+            )
+        };
+        arrow_data.extend_from_slice(&(bytes.len() as i64).to_le_bytes());
+        match compression {
+            Compression::LZ4 => {
+                compression::compress_lz4(bytes, arrow_data).unwrap();
+            }
+            Compression::ZSTD => {
+                compression::compress_zstd(bytes, arrow_data).unwrap();
+            }
+        }
+    } else {
+        todo!()
+    }
 }
 
 /// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
+#[inline]
 fn write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
     buffer: I,
     buffers: &mut Vec<Schema::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
+    compression: Option<Compression>,
 ) {
-    let len = buffer.size_hint().0 * std::mem::size_of::<T>();
-    let pad_len = pad_to_8(len as u32);
-    let total_len: i64 = (len + pad_len) as i64;
-    // assert_eq!(len % 8, 0, "Buffer width not a multiple of 8 bytes");
+    let start = arrow_data.len();
+
+    if let Some(compression) = compression {
+        _write_compressed_buffer_from_iter(buffer, arrow_data, is_little_endian, compression);
+    } else {
+        _write_buffer_from_iter(buffer, arrow_data, is_little_endian);
+    }
+
+    pad_buffer_to_8(arrow_data, arrow_data.len() - start);
+
+    let total_len = (arrow_data.len() - start) as i64;
     buffers.push(Schema::Buffer::new(*offset, total_len));
-
-    _write_buffer_from_iter(buffer, arrow_data, is_little_endian);
-
-    arrow_data.extend_from_slice(&vec![0u8; pad_len][..]);
     *offset += total_len;
 }
