@@ -23,9 +23,14 @@ use std::{
 use serde_derive::Deserialize;
 use serde_json::{json, Value};
 
-use crate::error::{ArrowError, Result};
+use crate::{
+    datatypes::UnionMode,
+    error::{ArrowError, Result},
+};
 
-use crate::datatypes::{get_extension, DataType, Field, IntervalUnit, Schema, TimeUnit};
+use crate::datatypes::{
+    get_extension, DataType, Field, IntegerType, IntervalUnit, Schema, TimeUnit,
+};
 
 pub trait ToJson {
     /// Generate a JSON representation
@@ -131,17 +136,20 @@ impl ToJson for Field {
             _ => vec![],
         };
         match self.data_type() {
-            DataType::Dictionary(ref index_type, ref value_type) => json!({
-                "name": self.name(),
-                "nullable": self.is_nullable(),
-                "type": value_type.to_json(),
-                "children": children,
-                "dictionary": {
-                    "id": self.dict_id(),
-                    "indexType": index_type.to_json(),
-                    "isOrdered": self.dict_is_ordered()
-                }
-            }),
+            DataType::Dictionary(ref index_type, ref value_type) => {
+                let index_type: DataType = (*index_type).into();
+                json!({
+                    "name": self.name(),
+                    "nullable": self.is_nullable(),
+                    "type": value_type.to_json(),
+                    "children": children,
+                    "dictionary": {
+                        "id": self.dict_id(),
+                        "indexType": index_type.to_json(),
+                        "isOrdered": self.dict_is_ordered()
+                    }
+                })
+            }
             _ => json!({
                 "name": self.name(),
                 "nullable": self.is_nullable(),
@@ -158,12 +166,58 @@ fn to_time_unit(item: Option<&Value>) -> Result<TimeUnit> {
         Some(p) if p == "MILLISECOND" => Ok(TimeUnit::Millisecond),
         Some(p) if p == "MICROSECOND" => Ok(TimeUnit::Microsecond),
         Some(p) if p == "NANOSECOND" => Ok(TimeUnit::Nanosecond),
-        _ => Err(ArrowError::Schema(
+        _ => Err(ArrowError::OutOfSpec(
             "time unit missing or invalid".to_string(),
         )),
     }
 }
 
+fn to_int(item: &Value) -> Result<IntegerType> {
+    Ok(match item.get("isSigned") {
+        Some(&Value::Bool(true)) => match item.get("bitWidth") {
+            Some(&Value::Number(ref n)) => match n.as_u64() {
+                Some(8) => IntegerType::Int8,
+                Some(16) => IntegerType::Int16,
+                Some(32) => IntegerType::Int32,
+                Some(64) => IntegerType::Int64,
+                _ => {
+                    return Err(ArrowError::OutOfSpec(
+                        "int bitWidth missing or invalid".to_string(),
+                    ))
+                }
+            },
+            _ => {
+                return Err(ArrowError::OutOfSpec(
+                    "int bitWidth missing or invalid".to_string(),
+                ))
+            }
+        },
+        Some(&Value::Bool(false)) => match item.get("bitWidth") {
+            Some(&Value::Number(ref n)) => match n.as_u64() {
+                Some(8) => IntegerType::UInt8,
+                Some(16) => IntegerType::UInt16,
+                Some(32) => IntegerType::UInt32,
+                Some(64) => IntegerType::UInt64,
+                _ => {
+                    return Err(ArrowError::OutOfSpec(
+                        "int bitWidth missing or invalid".to_string(),
+                    ))
+                }
+            },
+            _ => {
+                return Err(ArrowError::OutOfSpec(
+                    "int bitWidth missing or invalid".to_string(),
+                ))
+            }
+        },
+        _ => {
+            return Err(ArrowError::OutOfSpec(
+                "int signed missing or invalid".to_string(),
+            ))
+        }
+    })
+}
+
 fn children(children: Option<&Value>) -> Result<Vec<Field>> {
     children
         .map(|x| {
@@ -173,7 +227,9 @@ fn children(children: Option<&Value>) -> Result<Vec<Field>> {
                     .map(Field::try_from)
                     .collect::<Result<Vec<_>>>()
             } else {
-                Err(ArrowError::Schema("children must be an array".to_string()))
+                Err(ArrowError::OutOfSpec(
+                    "children must be an array".to_string(),
+                ))
             }
         })
         .unwrap_or_else(|| Ok(vec![]))
@@ -187,7 +243,7 @@ fn read_metadata(metadata: &Value) -> Result<BTreeMap<String, String>> {
                 match value.as_object() {
                     Some(map) => {
                         if map.len() != 2 {
-                            return Err(ArrowError::Schema(
+                            return Err(ArrowError::OutOfSpec(
                                 "Field 'metadata' must have exact two entries for each key-value map".to_string(),
                             ));
                         }
@@ -195,20 +251,20 @@ fn read_metadata(metadata: &Value) -> Result<BTreeMap<String, String>> {
                             if let (Some(k_str), Some(v_str)) = (k.as_str(), v.as_str()) {
                                 res.insert(k_str.to_string().clone(), v_str.to_string().clone());
                             } else {
-                                return Err(ArrowError::Schema(
+                                return Err(ArrowError::OutOfSpec(
                                     "Field 'metadata' must have map value of string type"
                                         .to_string(),
                                 ));
                             }
                         } else {
-                            return Err(ArrowError::Schema(
+                            return Err(ArrowError::OutOfSpec(
                                 "Field 'metadata' lacks map keys named \"key\" or \"value\""
                                     .to_string(),
                             ));
                         }
                     }
                     _ => {
-                        return Err(ArrowError::Schema(
+                        return Err(ArrowError::OutOfSpec(
                             "Field 'metadata' contains non-object key-value pair".to_string(),
                         ));
                     }
@@ -222,7 +278,7 @@ fn read_metadata(metadata: &Value) -> Result<BTreeMap<String, String>> {
                 if let Some(str_value) = v.as_str() {
                     res.insert(k.clone(), str_value.to_string().clone());
                 } else {
-                    return Err(ArrowError::Schema(format!(
+                    return Err(ArrowError::OutOfSpec(format!(
                         "Field 'metadata' contains non-string value for key {}",
                         k
                     )));
@@ -230,7 +286,7 @@ fn read_metadata(metadata: &Value) -> Result<BTreeMap<String, String>> {
             }
             Ok(res)
         }
-        _ => Err(ArrowError::Schema(
+        _ => Err(ArrowError::OutOfSpec(
             "Invalid json value type for field".to_string(),
         )),
     }
@@ -239,12 +295,12 @@ fn read_metadata(metadata: &Value) -> Result<BTreeMap<String, String>> {
 fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
     let type_ = item
         .get("name")
-        .ok_or_else(|| ArrowError::Schema("type missing".to_string()))?;
+        .ok_or_else(|| ArrowError::OutOfSpec("type missing".to_string()))?;
 
     let type_ = if let Value::String(name) = type_ {
         name.as_str()
     } else {
-        return Err(ArrowError::Schema("type is not a string".to_string()));
+        return Err(ArrowError::OutOfSpec("type is not a string".to_string()));
     };
 
     use DataType::*;
@@ -256,9 +312,9 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
         "fixedsizebinary" => {
             // return a list with any type as its child isn't defined in the map
             if let Some(Value::Number(size)) = item.get("byteWidth") {
-                DataType::FixedSizeBinary(size.as_i64().unwrap() as i32)
+                DataType::FixedSizeBinary(size.as_i64().unwrap() as usize)
             } else {
-                return Err(ArrowError::Schema(
+                return Err(ArrowError::OutOfSpec(
                     "Expecting a byteWidth for fixedsizebinary".to_string(),
                 ));
             }
@@ -269,13 +325,13 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
             // return a list with any type as its child isn't defined in the map
             let precision = match item.get("precision") {
                 Some(p) => Ok(p.as_u64().unwrap() as usize),
-                None => Err(ArrowError::Schema(
+                None => Err(ArrowError::OutOfSpec(
                     "Expecting a precision for decimal".to_string(),
                 )),
             };
             let scale = match item.get("scale") {
                 Some(s) => Ok(s.as_u64().unwrap() as usize),
-                _ => Err(ArrowError::Schema(
+                _ => Err(ArrowError::OutOfSpec(
                     "Expecting a scale for decimal".to_string(),
                 )),
             };
@@ -287,7 +343,7 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
             Some(p) if p == "SINGLE" => DataType::Float32,
             Some(p) if p == "DOUBLE" => DataType::Float64,
             _ => {
-                return Err(ArrowError::Schema(
+                return Err(ArrowError::OutOfSpec(
                     "floatingpoint precision missing or invalid".to_string(),
                 ))
             }
@@ -297,7 +353,9 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
             let tz = match item.get("timezone") {
                 None => Ok(None),
                 Some(Value::String(tz)) => Ok(Some(tz.clone())),
-                _ => Err(ArrowError::Schema("timezone must be a string".to_string())),
+                _ => Err(ArrowError::OutOfSpec(
+                    "timezone must be a string".to_string(),
+                )),
             }?;
             DataType::Timestamp(unit, tz)
         }
@@ -305,7 +363,7 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
             Some(p) if p == "DAY" => DataType::Date32,
             Some(p) if p == "MILLISECOND" => DataType::Date64,
             _ => {
-                return Err(ArrowError::Schema(
+                return Err(ArrowError::OutOfSpec(
                     "date unit missing or invalid".to_string(),
                 ))
             }
@@ -316,7 +374,7 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
                 Some(p) if p == 32 => DataType::Time32(unit),
                 Some(p) if p == 64 => DataType::Time64(unit),
                 _ => {
-                    return Err(ArrowError::Schema(
+                    return Err(ArrowError::OutOfSpec(
                         "time bitWidth missing or invalid".to_string(),
                     ))
                 }
@@ -331,87 +389,45 @@ fn to_data_type(item: &Value, mut children: Vec<Field>) -> Result<DataType> {
             Some(p) if p == "YEAR_MONTH" => DataType::Interval(IntervalUnit::YearMonth),
             Some(p) if p == "MONTH_DAY_NANO" => DataType::Interval(IntervalUnit::MonthDayNano),
             _ => {
-                return Err(ArrowError::Schema(
+                return Err(ArrowError::OutOfSpec(
                     "interval unit missing or invalid".to_string(),
                 ))
             }
         },
-        "int" => match item.get("isSigned") {
-            Some(&Value::Bool(true)) => match item.get("bitWidth") {
-                Some(&Value::Number(ref n)) => match n.as_u64() {
-                    Some(8) => DataType::Int8,
-                    Some(16) => DataType::Int16,
-                    Some(32) => DataType::Int32,
-                    Some(64) => DataType::Int64,
-                    _ => {
-                        return Err(ArrowError::Schema(
-                            "int bitWidth missing or invalid".to_string(),
-                        ))
-                    }
-                },
-                _ => {
-                    return Err(ArrowError::Schema(
-                        "int bitWidth missing or invalid".to_string(),
-                    ))
-                }
-            },
-            Some(&Value::Bool(false)) => match item.get("bitWidth") {
-                Some(&Value::Number(ref n)) => match n.as_u64() {
-                    Some(8) => DataType::UInt8,
-                    Some(16) => DataType::UInt16,
-                    Some(32) => DataType::UInt32,
-                    Some(64) => DataType::UInt64,
-                    _ => {
-                        return Err(ArrowError::Schema(
-                            "int bitWidth missing or invalid".to_string(),
-                        ))
-                    }
-                },
-                _ => {
-                    return Err(ArrowError::Schema(
-                        "int bitWidth missing or invalid".to_string(),
-                    ))
-                }
-            },
-            _ => {
-                return Err(ArrowError::Schema(
-                    "int signed missing or invalid".to_string(),
-                ))
-            }
-        },
+        "int" => to_int(item).map(|x| x.into())?,
         "list" => DataType::List(Box::new(children.pop().unwrap())),
         "largelist" => DataType::LargeList(Box::new(children.pop().unwrap())),
         "fixedsizelist" => {
             if let Some(Value::Number(size)) = item.get("listSize") {
                 DataType::FixedSizeList(
                     Box::new(children.pop().unwrap()),
-                    size.as_i64().unwrap() as i32,
+                    size.as_i64().unwrap() as usize,
                 )
             } else {
-                return Err(ArrowError::Schema(
+                return Err(ArrowError::OutOfSpec(
                     "Expecting a listSize for fixedsizelist".to_string(),
                 ));
             }
         }
         "struct" => DataType::Struct(children),
         "union" => {
-            let is_sparse = if let Some(Value::String(mode)) = item.get("mode") {
-                mode == "SPARSE"
+            let mode = if let Some(Value::String(mode)) = item.get("mode") {
+                UnionMode::sparse(mode == "SPARSE")
             } else {
-                return Err(ArrowError::Schema("union requires mode".to_string()));
+                return Err(ArrowError::OutOfSpec("union requires mode".to_string()));
             };
             let ids = if let Some(Value::Array(ids)) = item.get("typeIds") {
                 Some(ids.iter().map(|x| x.as_i64().unwrap() as i32).collect())
             } else {
-                return Err(ArrowError::Schema("union requires ids".to_string()));
+                return Err(ArrowError::OutOfSpec("union requires ids".to_string()));
             };
-            DataType::Union(children, ids, is_sparse)
+            DataType::Union(children, ids, mode)
         }
         "map" => {
             let sorted_keys = if let Some(Value::Bool(sorted_keys)) = item.get("keysSorted") {
                 *sorted_keys
             } else {
-                return Err(ArrowError::Schema("sorted keys not defined".to_string()));
+                return Err(ArrowError::OutOfSpec("sorted keys not defined".to_string()));
             };
             DataType::Map(Box::new(children.pop().unwrap()), sorted_keys)
         }
@@ -433,7 +449,7 @@ impl TryFrom<&Value> for Field {
                 let name = match map.get("name") {
                     Some(&Value::String(ref name)) => name.to_string(),
                     _ => {
-                        return Err(ArrowError::Schema(
+                        return Err(ArrowError::OutOfSpec(
                             "Field missing 'name' attribute".to_string(),
                         ));
                     }
@@ -441,7 +457,7 @@ impl TryFrom<&Value> for Field {
                 let nullable = match map.get("nullable") {
                     Some(&Value::Bool(b)) => b,
                     _ => {
-                        return Err(ArrowError::Schema(
+                        return Err(ArrowError::OutOfSpec(
                             "Field missing 'nullable' attribute".to_string(),
                         ));
                     }
@@ -459,7 +475,7 @@ impl TryFrom<&Value> for Field {
 
                 let type_ = map
                     .get("type")
-                    .ok_or_else(|| ArrowError::Schema("type missing".to_string()))?;
+                    .ok_or_else(|| ArrowError::OutOfSpec("type missing".to_string()))?;
 
                 let data_type = to_data_type(type_, children)?;
 
@@ -471,14 +487,14 @@ impl TryFrom<&Value> for Field {
 
                 let data_type = if let Some(dictionary) = map.get("dictionary") {
                     let index_type = match dictionary.get("indexType") {
-                        Some(t) => to_data_type(t, vec![])?,
+                        Some(t) => to_int(t)?,
                         _ => {
-                            return Err(ArrowError::Schema(
+                            return Err(ArrowError::OutOfSpec(
                                 "Field missing 'indexType' attribute".to_string(),
                             ));
                         }
                     };
-                    DataType::Dictionary(Box::new(index_type), Box::new(data_type))
+                    DataType::Dictionary(index_type, Box::new(data_type))
                 } else {
                     data_type
                 };
@@ -487,7 +503,7 @@ impl TryFrom<&Value> for Field {
                     let dict_id = match dictionary.get("id") {
                         Some(Value::Number(n)) => n.as_i64().unwrap(),
                         _ => {
-                            return Err(ArrowError::Schema(
+                            return Err(ArrowError::OutOfSpec(
                                 "Field missing 'id' attribute".to_string(),
                             ));
                         }
@@ -495,7 +511,7 @@ impl TryFrom<&Value> for Field {
                     let dict_is_ordered = match dictionary.get("isOrdered") {
                         Some(&Value::Bool(n)) => n,
                         _ => {
-                            return Err(ArrowError::Schema(
+                            return Err(ArrowError::OutOfSpec(
                                 "Field missing 'isOrdered' attribute".to_string(),
                             ));
                         }
@@ -508,7 +524,7 @@ impl TryFrom<&Value> for Field {
                 f.set_metadata(metadata);
                 Ok(f)
             }
-            _ => Err(ArrowError::Schema(
+            _ => Err(ArrowError::OutOfSpec(
                 "Invalid json value type for field".to_string(),
             )),
         }
@@ -548,13 +564,13 @@ fn from_metadata(json: &Value) -> Result<HashMap<String, String>> {
                 if let Value::String(v) = v {
                     Ok((k.to_string(), v.to_string()))
                 } else {
-                    Err(ArrowError::Schema(
+                    Err(ArrowError::OutOfSpec(
                         "metadata `value` field must be a string".to_string(),
                     ))
                 }
             })
             .collect::<Result<_>>(),
-        _ => Err(ArrowError::Schema(
+        _ => Err(ArrowError::OutOfSpec(
             "`metadata` field must be an object".to_string(),
         )),
     }
@@ -569,7 +585,7 @@ impl TryFrom<&Value> for Schema {
                 let fields = if let Some(Value::Array(fields)) = schema.get("fields") {
                     fields.iter().map(Field::try_from).collect::<Result<_>>()?
                 } else {
-                    return Err(ArrowError::Schema(
+                    return Err(ArrowError::OutOfSpec(
                         "Schema fields should be an array".to_string(),
                     ));
                 };
@@ -582,7 +598,7 @@ impl TryFrom<&Value> for Schema {
 
                 Ok(Self { fields, metadata })
             }
-            _ => Err(ArrowError::Schema(
+            _ => Err(ArrowError::OutOfSpec(
                 "Invalid json value type for schema".to_string(),
             )),
         }
