@@ -1,7 +1,18 @@
 //! Contains modules to interface with other formats such as [`csv`],
 //! [`parquet`], [`json`], [`ipc`], [`mod@print`] and [`avro`].
-#[cfg(any(feature = "io_csv_read", feature = "io_csv_write"))]
-#[cfg_attr(docsrs, doc(cfg(feature = "io_csv")))]
+#[cfg(any(
+    feature = "io_csv_read",
+    feature = "io_csv_read_async",
+    feature = "io_csv_write",
+))]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(
+        feature = "io_csv_read",
+        feature = "io_csv_read_async",
+        feature = "io_csv_write",
+    )))
+)]
 pub mod csv;
 
 #[cfg(feature = "io_json")]
