@@ -1,21 +1,17 @@
-use futures::{pin_mut, Stream, StreamExt};
 use parquet2::{
     encoding::{delta_length_byte_array, hybrid_rle, Encoding},
-    metadata::{ColumnChunkMetaData, ColumnDescriptor},
+    metadata::ColumnDescriptor,
     page::{BinaryPageDict, DataPage},
-    FallibleStreamingIterator,
 };
 
 use crate::{
-    array::{Array, Offset},
+    array::Offset,
     bitmap::{utils::BitmapIter, MutableBitmap},
     buffer::MutableBuffer,
-    datatypes::DataType,
-    error::{ArrowError, Result},
+    error::Result,
 };
 
 use super::super::utils;
-use super::utils::finish_array;
 
 /// Assumptions: No rep levels
 #[allow(clippy::too_many_arguments)]
@@ -233,7 +229,7 @@ pub(super) fn read_plain_required<O: Offset>(
     }
 }
 
-fn extend_from_page<O: Offset>(
+pub(super) fn extend_from_page<O: Offset>(
     page: &DataPage,
     descriptor: &ColumnDescriptor,
     offsets: &mut MutableBuffer<O>,
@@ -300,63 +296,3 @@ fn extend_from_page<O: Offset>(
     };
     Ok(())
 }
-
-pub fn iter_to_array<O, I, E>(
-    mut iter: I,
-    metadata: &ColumnChunkMetaData,
-    data_type: &DataType,
-) -> Result<Box<dyn Array>>
-where
-    ArrowError: From<E>,
-    O: Offset,
-    I: FallibleStreamingIterator<Item = DataPage, Error = E>,
-{
-    let capacity = metadata.num_values() as usize;
-    let mut values = MutableBuffer::<u8>::with_capacity(0);
-    let mut offsets = MutableBuffer::<O>::with_capacity(1 + capacity);
-    offsets.push(O::default());
-    let mut validity = MutableBitmap::with_capacity(capacity);
-    while let Some(page) = iter.next()? {
-        extend_from_page(
-            page,
-            metadata.descriptor(),
-            &mut offsets,
-            &mut values,
-            &mut validity,
-        )?
-    }
-
-    Ok(finish_array(data_type.clone(), offsets, values, validity))
-}
-
-pub async fn stream_to_array<O, I, E>(
-    pages: I,
-    metadata: &ColumnChunkMetaData,
-    data_type: &DataType,
-) -> Result<Box<dyn Array>>
-where
-    ArrowError: From<E>,
-    O: Offset,
-    E: Clone,
-    I: Stream<Item = std::result::Result<DataPage, E>>,
-{
-    let capacity = metadata.num_values() as usize;
-    let mut values = MutableBuffer::<u8>::with_capacity(0);
-    let mut offsets = MutableBuffer::<O>::with_capacity(1 + capacity);
-    offsets.push(O::default());
-    let mut validity = MutableBitmap::with_capacity(capacity);
-
-    pin_mut!(pages); // needed for iteration
-
-    while let Some(page) = pages.next().await {
-        extend_from_page(
-            page.as_ref().map_err(|x| x.clone())?,
-            metadata.descriptor(),
-            &mut offsets,
-            &mut values,
-            &mut validity,
-        )?
-    }
-
-    Ok(finish_array(data_type.clone(), offsets, values, validity))
-}
