@@ -1,6 +1,8 @@
 #![doc = include_str!("doc/lib.md")]
 // So that we have more control over what is `unsafe` inside an `unsafe` block
 #![allow(unused_unsafe)]
+//
+#![allow(clippy::len_without_is_empty)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 
 #[macro_use]
@@ -16,8 +18,6 @@ pub mod types;
 #[cfg(feature = "cache_aligned")]
 mod vec;
 
-#[cfg(feature = "compute")]
-#[cfg_attr(docsrs, doc(cfg(feature = "compute")))]
 pub mod compute;
 pub mod io;
 pub mod record_batch;
