@@ -111,6 +111,16 @@ fn equal(lhs: &dyn Scalar, rhs: &dyn Scalar) -> bool {
             let rhs = rhs.as_any().downcast_ref::<ListScalar<i64>>().unwrap();
             lhs == rhs
         }
-        _ => unimplemented!(),
+        DataType::Dictionary(key_type, _) => match_integer_type!(key_type, |$T| {
+            let lhs = lhs.as_any().downcast_ref::<DictionaryScalar<$T>>().unwrap();
+            let rhs = rhs.as_any().downcast_ref::<DictionaryScalar<$T>>().unwrap();
+            lhs == rhs
+        }),
+        DataType::Struct(_) => {
+            let lhs = lhs.as_any().downcast_ref::<StructScalar>().unwrap();
+            let rhs = rhs.as_any().downcast_ref::<StructScalar>().unwrap();
+            lhs == rhs
+        }
+        other => unimplemented!("{}", other),
     }
 }
