@@ -3,6 +3,13 @@ use arrow2::{
     buffer::MutableBuffer,
 };
 
+#[test]
+fn from_slice() {
+    let slice = &[true, false, true];
+    let a = MutableBitmap::from(slice);
+    assert_eq!(a.iter().collect::<Vec<_>>(), slice);
+}
+
 #[test]
 fn trusted_len() {
     let data = vec![true; 65];
