@@ -87,7 +87,7 @@ fn write_null() -> Result<()> {
 #[test]
 fn write_dictionary() -> Result<()> {
     // define a schema.
-    let field_type = DataType::Dictionary(Box::new(DataType::Int32), Box::new(DataType::Utf8));
+    let field_type = DataType::Dictionary(i32::KEY_TYPE, Box::new(DataType::Utf8));
     let schema = Arc::new(Schema::new(vec![Field::new("d1", field_type, true)]));
 
     let mut array = MutableDictionaryArray::<i32, MutableUtf8Array<i32>>::new();
@@ -119,7 +119,7 @@ fn write_dictionary() -> Result<()> {
 #[test]
 fn dictionary_validities() -> Result<()> {
     // define a schema.
-    let field_type = DataType::Dictionary(Box::new(DataType::Int32), Box::new(DataType::Int32));
+    let field_type = DataType::Dictionary(i32::KEY_TYPE, Box::new(DataType::Int32));
     let schema = Arc::new(Schema::new(vec![Field::new("d1", field_type, true)]));
 
     let keys = PrimitiveArray::<i32>::from([Some(1), None, Some(0)]);
@@ -391,7 +391,7 @@ fn write_union() -> Result<()> {
         Field::new("a", DataType::Int32, true),
         Field::new("b", DataType::Utf8, true),
     ];
-    let data_type = DataType::Union(fields, None, true);
+    let data_type = DataType::Union(fields, None, UnionMode::Sparse);
     let types = Buffer::from(&[0, 0, 1]);
     let fields = vec![
         Arc::new(Int32Array::from(&[Some(1), None, Some(2)])) as Arc<dyn Array>,
